/*
  ķ
                                                                         
   Program...: HYPER.PRG                                                 
   Author....: Team SuccessWare                                          
   Notice....: Copyright 1993-95 - SuccessWare 90, Inc.                  
   Purpose...: Demonstrates using the SIx Driver's multi-key indexes     
               for a hypertext help / search system.                     
                                                                         
   Compile...: CLIPPER hyper /n/w /dCDX  (for SIXCDX)                    
           or: CLIPPER hyper /n/w /dNSX  (for SIXNSX)                    
                                                                         
   Link......: BLINKER FI hyper, sixtools, sixcdx @six3  (for SIXCDX)    
           or: BLINKER FI hyper, sixtools, sixnsx @six3  (for SIXNSX)    
                                                                         
   NOTES:  This program will create its own database and index file if   
           they don't exist already.  Add text to the memo fields and    
           then add keywords to use for the hypertext search by posi-    
           tioning the cursor on the word to be added and pressing the   
           <F7> key.  Be sure that you have saved any new text edits by  
           pressing the <F2> key before attempting to add a new keyword. 
                                                                         
  Ľ
*/

#ifdef CDX
  #include "SIXCDX.CH"
  #define INDEXEXT ".CDX"
#else
  #include "SIXNSX.CH"
  #define INDEXEXT ".NSX"
#endif

#include "inkey.ch"
#include "memoedit.ch"

#xtranslate String( <nVal> )  =>  AllTrim( Str( <nVal> ))
#xtranslate Box( <nUR>, <nUC>, <nLR>, <nLC> ) 	 => ;
            Scroll( <nUR>+1, <nUC>+1, <nLR>-1, <nLC>-1 );
           ;DispBox( <nUR>, <nUC>, <nLR>, <nLC>, 1 )

EXTE sxChar

STATIC lAllDone := .f., lFound := .f., cSearchStr := ""

FUNC Hyper()

LOCAL aFields := {}, nCount, cOldColor := SetColor(), cTempStr := ""
LOCAL nOldRec
//MEMVAR GetList
FIELD NOTES

SET SCOREBOARD off
SetKey( K_F1, {|| Help() } )
SetColor("w+/b,b/w,,,b/w")
Scroll()

// Create HYPER.DBF if it doesn't exist; Open HYPER.DBF
if !file("HYPER.DBF")
  CLS
  ? "Creating Data and Index Files..."
  aFields := {{"TOPIC", "C",40,0},;
              {"NOTES","M",10,0}}
  dbCreate( "HYPER", aFields )
endif
use HYPER excl

// Append a blank record if DBF is empty
if lastrec() == 0
  dbAppend()
endif

// Create HYPER index if it doesn't exist
if !file("HYPER" + INDEXEXT)
  index on sxChar(15) to HYPER
endif
set index to HYPER
dbGoto(1)

// Display Screen
@ 1,0 to 21,79 double color "bg+/b"
@ 0,0 say " Topic: " + replicate( " ", 72 ) color "b/w"

SetColor("b/w")
@22, 3 say " <F2> Save Memo Edits  "
@23, 3 say " <F3> Keyword Search   "
@24, 3 say " <F4> Continue Search  "
@22,28 say " <F5> Previous Record  "
@23,28 say " <F6> Next Record      "
@24,28 say " <F7> Add Keyword      "
@22,53 say " <F8> Delete Keyword   "
@23,53 say " <F9> Edit Topic Field "
@24,53 say " <Esc>     QUIT        "
SetColor("w+/b,b/w,,,b/w")

// Main loop
lAllDone := .f.
do while .t.
  do while !lAllDone
    nOldRec := RecNo()
    cTempStr := MemoEdit( NOTES,2,1,20,78,.t.,"MemoUDF" )
    if !lAllDone .and. nOldRec == RecNo()
      hyper->NOTES := cTempStr
    endif
  enddo
  if MyAlert("Quit?", {"Yes", "No "}) == 1
    exit
  else
    lAllDone := .f.
  endif
enddo
SetColor( "w/n" )
CLS

Return( NIL )

/*-------------------------( GetWord Function )-------------------------*/

FUNC GetWord( nRow, nCol, nLen )

LOCAL cStr := "", nCount

for nCount := 1 to nLen
  cStr += left( SaveScreen( nRow, nCol+nCount-1, nRow, nCol+nCount-1 ), 1 )
next

Return( cStr )


/*-------------------------( MemoUDF Function )-------------------------*/

FUNC MemoUDF( nMode, nRow, nCol )

LOCAL cKey, nByte, cScreen, cTempStr, aRowCol, nTRow, nTCol
LOCAL GetList := {}
FIELD NOTES

if nMode == ME_IDLE
  if lFound
    lFound := .F.
    cTempStr := upper( AllTrim( cSearchStr ))
    aRowCol := mpostolc( NOTES, 78, at( cTempStr, upper(NOTES)))
    nTRow := aRowCol[1] + 1
    nTCol := aRowCol[2] + 1
    cTempStr := GetWord( nTRow, nTCol, len( cTempStr ))
    @ nTRow, nTCol say cTempStr color "w+/br"
    DevPos( nTRow, nTCol )
  endif
endif

if nMode == ME_UNKEY .or. nMode == ME_UNKEYX

  cKey := LastKey()

  do case
    case cKey == K_ESC
      lAllDone := .T.
      return K_ESC

    case cKey == K_F2
      return K_CTRL_W

    case cKey == K_F3
      return KeySearch(.f.)

    case cKey == K_F4
      return KeySearch(.t.)

    case cKey == K_F5
      return PrevRec()

    case cKey == K_F6
      return NextRec()

    case cKey == K_F7

      // Get current byte in memo field
      nByte := mlctopos( NOTES, 78, nRow, nCol )

      // Save everything in memo field up to this byte to temporary string
      cTempStr := SubStr( NOTES, 1, nByte )

      // Check if cursor was on a space or carriage return (EOL)
      if right( cTempStr, 1 ) $ chr(32)+chr(13)
        Return( ME_DEFAULT)
      endif

      // Convert carriage returns to spaces
      cTempStr := StrTran( cTempStr, chr(13), chr(32))

      // Locate last space in the temporary string; Gets first byte of keyword
      nByte := rat( " ", cTempStr ) + 1

      // Save rest of memo field to temp string; add a space at the end
      cTempStr := SubStr( NOTES, nByte ) + " "

      // Add Key To Index
      SaveKey( cTempStr )

    case cKey == K_F8
      KeyDel()

    case cKey == K_F9
      @ 0, 8 get hyper->TOPIC
      read

  endcase
endif
@ 0, 8 say padr( hyper->TOPIC, 40 ) color "b/w"

Return( ME_DEFAULT )

/*-------------------------( PrevRec Function )-------------------------*/

FUNC PrevRec()

dbGoto( Recno()-1 )
if bof()
  tone(200,1)
  dbGoTo(1)
endif

Return( K_ESC )

/*-------------------------( NextRec Function )-------------------------*/

FUNC NextRec()

LOCAL cTopic := hyper->TOPIC, nOption

dbGoto( Recno()+1 )

// If at EOF and previous record was not empty, add a new record
if eof()
  if !empty( cTopic )
    nOption := MyAlert( "Bottom of file!  Create New Record?", {"YES", "No"} )
    if nOption == 1
      dbAppend()
      Return( K_ESC )
    endif
  endif
  dbGoTo( LastRec())
endif

Return( K_ESC )

/*------------------------( KeySearch Function )------------------------*/

FUNC KeySearch( lCont )

LOCAL nOldRec := RecNo(), cOldColor := SetColor(), cTempStr := space(15)
LOCAL GetList := {}
STATIC nLastKey := 0

// Was this a continuation?
if lCont
  sx_KeyGoto( nLastKey )
  sx_KeySkip()
  if eof() .or. ( sx_KeyData() # cSearchStr )
    MyAlert( "No more matches for: " + cSearchStr )
    dbGoto( nOldRec )
    lFound := .f.
  else
    lFound := .t.
  endif
  nLastKey := sx_KeyNo()
  Return( K_CTRL_W )
endif

SetColor("b/w")
@ 0,49 say "Keyword Search:" get cTempStr picture "@!"
read
cSearchStr := AllTrim( cTempStr )

if LastKey() # K_ESC
  if dbSeek( cSearchStr )
    lFound := .t.
    __KillRead()
  else
    MyAlert( "Not Found!" )
    lFound := .f.
    dbGoto( nOldRec )
  endif
  nLastKey := sx_KeyNo()
endif
SetColor( cOldColor )

return( K_CTRL_W )

/*-------------------------( SaveKey Function )-------------------------*/

FUNC SaveKey( cTempStr )

LOCAL cWord, nOldRec := RecNo()

// Extract entire key word (should be the first word in cTempStr)
cWord := upper( left( cTempStr, at( " ", cTempStr ) - 1 ))

if empty( cWord )
  Return( NIL )
endif

// Strip off any leading quotes, parens, brackets, or curly braces
do while !isalpha( left( cWord, 1 ))
  cWord := SubStr( cWord, 2 )
enddo

// Strip off any ending punctuation marks (,.!?) or quotes and parens
do while !isalpha( right( cWord, 1 ))
  cWord := left( cWord, len( cWord ) - 1 )
enddo

// Make sure cWord is 15 characters or less; trim off extra spaces
cWord := trim( padr( cWord, 15 ))

if Sx_KeyAdd( NIL, 1, sxChar( 15, cWord ))
   MyAlert( "Keyword " + cWord + " added!" )
else
   MyAlert( "Keyword " + cWord + " was NOT added!" )
endif

Return( NIL )

/*--------------------------( KeyDel Function )-------------------------*/

FUNC KeyDel()

LOCAL nKeyPos, aKeys[1], aKeyNums[1], nChoice, cScreen,;
      cOldColor := SetColor(), nRecNo := Recno()

if Sx_FindRec( nRecNo )
  aKeys[1]    := " " + Sx_KeyData()
  aKeyNums[1] := Sx_KeyNo()
else
  MyAlert( "No keywords found for record #" + String( Recno()))
  Return( NIL )
endif

do while Sx_FindRec( nRecNo, .t. )
  AAdd( aKeys, " " + Sx_KeyData())
  AAdd( aKeyNums, Sx_KeyNo())
end

cScreen := SaveScreen( 6,56,13,77 )
SetColor("br+/n")
Box( 6,56,13,77 )
@ 6,61 say " Keywords "
nKeyPos := aChoice( 7,57,12,76,aKeys )

if nKeyPos # 0
  Sx_KeyGoto( aKeyNums[ nKeyPos ] )
  nChoice := MyAlert( "Delete keyword?: " + trim( Sx_KeyData()), {"Yes", "No"} )
  if nChoice == 1
    Sx_KeyDrop()
  endif
endif

dbGoto( nRecNo )
SetColor( cOldColor )
RestScreen( 6,56,13,77,cScreen )

Return( NIL )

/*---------------------------( Help Function )--------------------------*/

PROC Help()

MyAlert( " <F2> - Save Memo Contents  ;" +;
         " <F3> - Search For Keyword  ;" +;
         " <F4> - Continue Search     ;" +;
         " <F5> - Previous Record     ;" +;
         " <F6> - Next Record         ;" +;
         " <F7> - Add Keyword         ;" +;
         " <F8> - Delete Keyword      ;" +;
         " <F9> - Edit Topic Contents ;" +;
         " <Esc>- QUIT                " )
Return

