/*
͸
 Source file: INIDEMO.PRG                                                 
 Description: Sample Code Demonstrating Use Of .INI Files                 
 Written by : Team SuccessWare                                            
 Notice     : Copyright 1993-95 - SuccessWare 90, Inc.                    
                                                                          
  Compile:  CLIPPER inidemo /n/m/w /dCDX   (for SIXCDX)                   
       or:  CLIPPER inidemo /n/m/w /dNSX   (for SIXNSX)                   
  Link   :  BLINKER FI inidemo, sixtools @six3                            
                                                                          
  NOTES:  This program requires a TEST.DBF file of any size, which must   
          be created with our DBCRE8.EXE utility.  For example, to        
          create a TEST.DBF file with 1000 records, you would type this   
          at the DOS command-line:                                        
                                                                          
              C:\> DBCRE8 1000                                            
                                                                          
;
  
*/

#ifdef CDX
	#include "SIXCDX2.CH"
	#define RDDNAME "SIXCDX"
	#define MEMOEXT ".FPT"
	#define INDEXEXT ".CDX"
#else
	#include "SIXNSX2.CH"
	#define RDDNAME "SIXNSX"
	#define MEMOEXT ".SMT"
	#define INDEXEXT ".NSX"
#endif

#include "box.ch"
#include "Memoedit.ch"
#include "inkey.ch"

#xtrans  writeString(<string>,<handle>)  => ;
         fwrite(<handle>,(<string>),len(<string>) ) == len(<string>)
#xtrans  isTrue(<cStr>)      => ("*"+upper(<cStr>)+"*" $ "*.T.*TRUE*ON*YES*" )
#xtrans  feraseIf(<cFile>)   => iif(file(<cFile>), ferase(<cFile>), 0)
#define CRLF  CHR(13) + CHR(10)

#define    TOPTOP    1
#define    TOPBOT    (maxrow() - 9)

#define    LOWERTOP  (TOPBOT + 1)
#define    LOWERBOT  (maxrow()-1)

#define    MID       (int(maxcol()/2))
#define    FAR_RIGHT (maxcol()-1)
#define    FAR_LEFT  1

// top box coords
#define TT  TOPTOP
#define TL  FAR_LEFT
#define TB  TOPBOT
#define TR  FAR_RIGHT

// INI window coords
#define IT  LOWERTOP
#define IL  MID
#define IB  LOWERBOT
#define IR  FAR_RIGHT

// msgbox coords
#define MT  LOWERTOP
#define ML  FAR_LEFT
#define MB  LOWERBOT
#define MR  (MID - 1)

//
function iniDemo()
   local bigscreen := savescreen()
	local oldColor := setcolor("B+/W*")
	local oldcurs := setcursor(0)
	local oldBlink := setBlink(.f.)
	local handle

	// prepare demo files
	if demoPrep()
		// demo begin
      dispbox( 0,0, MT - 2, maxcol(), B_SINGLE+" ","B+/N" )
      dispbox( MT-1, 0, maxrow(), maxcol(), B_SINGLE+" ","B+/N" )

      msgBox({"An INI file can be automatically",;
              "read by the SIx Driver for each",;
              "table in your application."},.t., .t. )

      msgBox({"Here is how you can use INI files",;
              "with the TRIGGER system to easily",;
              "implement data dictionary features."},.t., .t.)

      msgBox({"First we will try to open a table",;
              "that does NOT exist!"},.t., .t.)

		use DBF_GONE via RDDNAME TRIGGER "TESTTRIGGER"
		DBF_GONE->(dbclosearea())

      msgBox({"The next table we will open DOES",;
              "exist. BUT it has no INI file! We'll" ,;
              "catch this in the POST_USE EVENT..." },.t., .t.)

		use NEW_INI via RDDNAME TRIGGER "TESTTRIGGER"

		NEW_INI->(dbclosearea())

      msgBox({"Now we will open a table that,"   ,;
              "according to the INI file,"       ,;
              "is missing the index files. Again",;
              "the POST_USE EVENT catches this. "},.t., .t.)

      use NDX_GONE via RDDNAME TRIGGER "TESTTRIGGER"

      NDX_GONE->(dbclosearea())

      msgBox({""}, .f., .t.)

	endif

	setBlink(oldBlink)

	set(_SET_CURSOR, oldcurs)

	setcolor(oldColor)

	restscreen(bigscreen)
return NIL

/*
 Chicken and/or egg:
  1) if we have ini file and no table, write table.
  2) if we have table and no ini files, write ini file.
  3) if we have neither, your guess is as good as mine.
*/
function testTrigger(nEvent, nArea, nFieldPos, xTrigVal)
	do case
   case nEvent == EVENT_PREUSE
		_sx_INIInit( nArea )

      rddsetdefault( RDDNAME )

      msgBox({ "EVENT_PREUSE was triggered:"},.t., .t.)

      if !file(xTrigVal+".DBF") .and. file(xTrigVal+".INI")
         msgBox({ xTrigVal + ".DBF" +" table was not found!",;
                  "Create it from the INI file!"} ,.f., .f.)
         showIni(xTrigVal+".INI")
         dbCreate( xTrigVal, iniFields())
      else
         msgBox({ "About to open " + xTrigVal + ".DBF" } ,.t., .f.)
      endif

	case nEvent == EVENT_POSTUSE
      msgBox({"EVENT_POSTUSE was triggered:"},.t., .t.)

      if !file( strtran( sx_tableName(), ".DBF", ".INI"))

         msgBox({"INI file for " + sx_tableName()  ,;
                 "is missing! Creating INI file"   ,;
                 "from DBF..."}, .t., .f.)

         createIni()                                 // if we don't have an ini file, write one
      endif

      idxCheck()                                 // check index tags

      msgBox({"Browse newly opened file...",;
                 "(<ESC> to end browse)"}, .f., .f.)
      demoBrowse()
	endcase
Return( .T. )

//
static function demoBrowse()
   local oldscreen := savescreen(TT, TL, TB, TR)
   dispbox(TT, TL, TB - 2, TR, B_SINGLE+" ")
   @ TT, TL+2 say sx_tablename()
   browse(TT+1, TL+1, TB-3, TR-1)
   restscreen(TT,TL,TB,TR, oldscreen)
return nil

//
function iniFields()
	local aFields_ := sx_iniHeader( "FIELDS" )
	local nLen    	:= len(aFields_), i, aStru := {}
	for i := 1 to nLen
		aadd(aStru, parseFieldInfo(aFields_[i,1]+"," + aFields_[i,2]))
	next
return aStru

//
static function parseFieldInfo(cString)
	local aTokens_ := {}, nPtr
	cString  := ALLTRIM(cString)

   do  while (nPtr := at(",", cString)) != 0
		aadd(aTokens_, iif(nPtr > 1, rtrim(left(cString, nPtr-1)), ""))
		cString := iif(len(cString) > nPtr, ltrim(substr(cString, nPtr+1)), "")
	enddo

	aadd(aTokens_, cString)
	asize(aTokens_, 4)

	do case
	case  aTokens_[2] =="C"
		aTokens_[3] := val(aTokens_[3])
		aTokens_[4] := 0

	case  aTokens_[2] == "N"
		aTokens_[3] := val(aTokens_[3])
		if empty(aTokens_[4])
			aTokens_[4] := 0
		else
			aTokens_[4] := val(aTokens_[4])
		endif

	case  aTokens_[2] == "D"
		aTokens_[3] := 8
		aTokens_[4] := 0

	case  aTokens_[2] == "L"
		aTokens_[3] := 1
		aTokens_[4] := 0

	case  aTokens_[2] == "M"
		aTokens_[3] := 10
		aTokens_[4] := 0
	endcase
return aTokens_

//
function createIni()
	local handle, aStru_, aIdx_, cTmp, nBytes, i, nlen
	local cFile := strtran( sx_tableName(), ".DBF" )

		handle := fcreate(cFile+".INI")

		if handle > 0
			aSTru_  := dbstruct()
			nLen    := len(aStru_)
			nBytes  := writeString("[FIELDS]"+ CRLF , handle)
			for i := 1 to nLen
				nBytes := writeString(;
					padr(aStru_[i,1]    ,10) + " = "+;
					aStru_[i,2]        + " , "+;
               str( aStru_[i,3], 5 ) + " , "+;
               str( aStru_[i,4], 5 ) +  CRLF , handle)
			next
		endif
		fclose(handle)
		showIni(cFile+".INI")
return nil

//
function sx_IniValue( cHeader, cKey )
	local  nKey
   local aInfo_ := sx_iniHeader(cHeader)
	if !empty(aInfo_)
		if ( nKey := ascan(aInfo_,{|a| valtype(a[1]) == "C" .and. upper(a[1]) == upper(cKey)}) ) > 0
			retu aInfo_[nKey,2]
		endif
	endif
return ""

//
function idxCheck()
	local aTags_ 	:= sx_iniHeader( "INDEX" )
	local nLen 		:= len( aTags_ )
	local cPrefix 	:= strtran(sx_tableName(),".DBF")
	local asxTags_, i

	// order must be set for sx_tags to return something
	// or to do an sx_killtag()
	ordSetFocus(1)
	asxtags_ := sx_tags()
	if isTrue(sx_iniValue("GENERAL", "FORCE_REINDEX"))
      sx_killtag(.t.)
   endif
	if !empty(nLen)
		if empty(asxTags_)
			asxtags_ := array(nLen)
		else
			asize(asxtags_, nLen)
		endif
		if file(cPrefix+".INI")
         msgBox({"Indexing information is missing!",;
                 "Recreating indexes from information",;
                 "stored in INI file..."}, .f., .f.)

         showIni(cPrefix+".INI", .t.)

			for i := 1 to nLen
				if ascan( aSxTags_, {|a| valtype(a)=="C" .and. upper(a)==upper( aTags_[i,1] )} ) == 0
					// temporarily disable trigger so index creation isn't slowed down
               sx_setTrigger( TRIGGER_DISABLE )
					if lastrec()<=10
						index on (aTags_[i,2]) tag (aTags_[i,1])
					else
						index on (aTags_[i,2]) tag (aTags_[i,1]) eval odometer() every 10
					endif
               sx_setTrigger( TRIGGER_ENABLE )
				endif
			next
         showIni()
         msgBox({" "},.t., .f.)
		endif
	endif
return nil

//
static function showIni(cFile, lKeep)
	local cStr := memoread(cfile), cTmp, nat, n
	local oldcolor := setcolor("W+/B")
   static oldScr

   if oldScr == NIL
      oldScr := saveScreen(IT,IL,IB,IR)
   endif

   if lKeep == NIL
      lKeep := .f.
   endif

   if !empty( cFile )
      dispbox(IT,IL,IB,IR,B_SINGLE+" ")
      @ IT, IL+2 say cfile
      for n := 1 to IB-IT-1
         cTmp := substr(cStr,1, (nat := at(CRLF,cStr))-1 )
         if !empty(cTmp)
            @IT +n, IL+1 say cTmp
         else
            exit
         endif
         cSTr := substr(cStr, nat+2)
      next
   endif
	setcolor(oldColor)
   if !lKeep
      inkey(2)
      restscreen( IT,IL,IB,IR, oldScr)
   endif
return nil

//
static function demoPrep()
   local handle, ok := .t., oldColor := setColor()
   if file("TEST.DBF")
      copy file TEST.DBF to NDX_GONE.DBF
		copy file TEST.DBF to NEW_INI.DBF
   else
      setcolor("W+/R")
      ? "TEST.DBF not found!  Use the DBCRE8.EXE utility "
      ? "to create one before running this program.      "
      ? "For example:                                    "
      ? "               DBCRE8 1000                      "
      setcolor(oldColor)
      ?
      QUIT
   ENDIF
	if file("TEST"+MEMOEXT)
      copy file ("TEST"+MEMOEXT) to ("NDX_GONE"+MEMOEXT)
		copy file TEST.DBF to NEW_INI.DBF
	endif
   BEGIN SEQUENCE
		assert( feraseIf("DBF_GONE.INI") <> -1 )
		assert( feraseIf("DBF_GONE.DBF") <> -1 )
		assert( feraseIf("DBF_GONE"+MEMOEXT) <> -1 )
		assert( feraseIf("DBF_GONE"+INDEXEXT) <> -1 )
      assert( feraseIf("NDX_GONE.INI") <> -1 )
      assert( feraseIf("NDX_GONE"+INDEXEXT) <> -1 )
		assert( feraseIf("NEW_INI.INI")  <> -1 )
		assert((handle := fcreate("DBF_GONE.INI")) <> -1)
		assert( writeString("[FIELDS]"+CRLF, handle))
		assert( writeString("FIRST  = C, 20 , 0"+CRLF, handle))
		assert( writeString("LAST   = C, 20 , 0"+CRLF, handle))
		assert( writeString("ZIP    = C, 10 , 0"+CRLF, handle))
		assert( writeString("SALARY = N,  6 , 2"+CRLF, handle))
		assert( writeString("NOTES  = M, 10 , 0"+CRLF, handle))
		assert( writeString( CRLF, handle))
		fclose(handle)
      assert((handle := fcreate("NDX_GONE.INI")) <> -1)
		assert( writeString("[INDEX]"+CRLF, handle))
		assert( writeString("Name = upper(last) + upper(first)"+CRLF, handle))
		assert( writeString("Time4Raze = salary < 10000"+CRLF, handle))
		assert( writeString("ZIP = zip"+CRLF, handle))
		assert( writeString("[GENERAL]"+CRLF, handle))
		assert( writeString("FORCE_REINDEX = YES"+CRLF, handle))
		fclose(handle)
	RECOVER
		? "file write or file erase error"
		ok := .f.
	END SEQUENCE
	if file("TEST.DBF")
      copy file TEST.DBF to NDX_GONE.DBF
		copy file TEST.DBF to NEW_INI.DBF
	endif
	if file("TEST"+MEMOEXT)
      copy file ("TEST"+MEMOEXT) to ("NDX_GONE"+MEMOEXT)
		copy file TEST.DBF to NEW_INI.DBF
	endif
return  ok

//
static function assert(expr)
	if !expr
		BREAK
	endif
return nil

//
static function msgBox( aMsg, lVerify, lKillold )
   LOCAL cColor := SetColor("bg+/n")
   LOCAL nLen := len(aMsg), i

   STATIC aStr := {}

   @ MT,ML CLEAR TO MB,MR

   if lKillold
      aStr := {}
   endif
   for i := 1 to nLen
      aadd(aStr,aMsg[i])
   next
   nLen := len( aStr )
	for i := 1 to nLen
      @ MT + i, 2 SAY padr(aStr[i], MR - ML - 1) COLOR "gr+/n"
	next
	IF lVerify
		@ MB,2  say "<press key>" color "R+/n"
		inkey(0)
		@ MB ,2  say "           " color "n/n"
	ENDIF
	SetColor( cColor )
Return NIL

