/*
͸
 Source file: M6DEMO.PRG  (For use with SIx Driver v3.0 ONLY)             
 Description: Sample Code to demonstrate Mach SIx query optimizer         
 Written by : Team SuccessWare                                            
 Notice     : Copyright 1992-95 - SuccessWare 90, Inc.                    
                                                                          
  Compile:  CLIPPER m6demo /n/w /dCDX   (for SIXCDX)                      
       or:  CLIPPER m6demo /n/w /dNSX   (for SIXNSX)                      
                                                                          
  Link:  BLINKER FI m6demo,sixtools @six3                                 
                                                                          
 NOTE:  Requires TEST5K.DBF (5000 records) and TEST.DBF (1000 records).   
        These files should be created using the DBCRE8.EXE utility, which 
        is included with the SIx Driver.                                  
                                                                          
          C:> DBCRE8 5000 /fTEST5K                                        
          C:> DBCRE8 1000                                                 
                                                                          
;
  
*/


#ifdef CDX
  #include "SIXCDX2.CH"
  #define RDDNAME "SIXCDX"
  #define MEMOEXT ".FPT"
  #define INDEXEXT ".CDX"
#else
  #include "SIXNSX2.CH"
  #define RDDNAME "SIXNSX"
  #define MEMOEXT ".SMT"
  #define INDEXEXT ".NSX"
#endif

#include "MACHSIX.CH"

#command COUNT FOR <for>                                                ;
 =>                                                                     ;
         TestCount( <"for">, <{for}> )

STATIC lMachPass := .f.

FUNC M6Demo()

LOCAL nCount := 0, nSal := 0, nAvg := 0, nRow := 0, nCol := 0
LOCAL cOldColor := SetColor("bg+/b"), nMaxCol := MaxCol()
LOCAL nStart := 0, nEnd := 0, nOpti := 0, cRank := "", nPass:=0
FIELD First,Last,Age,Salary,State,MArried,HireDate

SET DELETED OFF
SET CURSOR OFF

//..............................................................open databases
CLOSE ALL
fErase( "TESTX" + INDEXEXT )
USE test5k SHARED NEW ALIAS testntx VIA 'DBFNTX'
USE test5k SHARED NEW ALIAS testsix VIA RDDNAME
nCount := RecCount()/20

CLS

//..............................................................create indexes
if !file( "TESTX" + INDEXEXT  )
  ? "Making TESTX" + INDEXEXT
  INDEX ON last+first TAG last     OF testx OPTION Odometer() STEP nCount
  INDEX ON state      TAG state    OF testx OPTION Odometer() STEP nCount
  INDEX ON age        TAG age      OF testx OPTION Odometer() STEP nCount
  INDEX ON married    TAG married  OF testx OPTION Odometer() STEP nCount
  INDEX ON hiredate   TAG hiredate OF testx OPTION Odometer() STEP nCount
  INDEX ON salary     TAG salary   OF testx OPTION Odometer() STEP nCount
endif

//............................................select testsix, open TESTX index
SELE testsix
SET INDEX TO ("TESTX" + INDEXEXT)
SET ORDER TO 0

CLS

//..................................................display fancy title screen
@ 0, 0 SAY Repl( "", nMaxCol+1 ) COLOR "bg/b"
@ 1,25 SAY "Mach SIx Demo - "+STR(RecCount(),6)+" Records" COLOR "w+/b"
@ 2, 0 SAY Repl( "", nMaxCol+1 ) COLOR "bg/b"

//.....Make two passes through this COUNT screen: 1st - Clipper; 2nd - MachSIx
lMachPass := .f.
For nPass := 1 to 2

  if nPass == 1
    MyAlert( "First we'll run through some basic COUNT operations  ;" +;
             "in pure Clipper (w/ Mach SIx optimizations disabled)." )
  else
    MyAlert( "Now, we'll let Mach SIx optimize the same COUNT;" +;
             "commands and see how it fares.  Don't blink!   " )
  endif

  @ 4,5 say "   [ Pure CLIPPER ]              [ Optimized w/ Mach SIx ]" COLOR "w+/b"
  @ 5,5 say " --------------------------    ------------------------------"

  //...............................................simple optimized expression
  ? "[ Simple Expressions ]:"
  ?
  COUNT FOR STATE = "CA"
  COUNT FOR AGE = 21
  COUNT FOR LAST = "Smith"
  COUNT FOR HIREDATE = CTOD("01/24/83")
  COUNT FOR MARRIED

  lMachPass := .t.

next nPass

//................................wait for key, clear screen, reset cursor pos
?
wait "Press any key to continue.                           "
Scroll(3,0,24,79)

lMachPass := .f.

For nPass := 1 to 2

  if nPass == 1
    MyAlert( "Next, we'll run through some more complex;" +;
             "operations.  First in pure Clipper...    " )
  else
    MyAlert( "Now, we'll let Mach SIx optimize the same operations..." )
  endif

  @ 4,5 say "   [ Pure CLIPPER ]              [ Optimized w/ Mach SIx ]" COLOR "w+/b"
  @ 5,5 say " --------------------------    ------------------------------"

  //...................................................mixed field, .and./.or.
  ? "[ Mixed Field, .OR./.AND. ]:"
  ?
  COUNT FOR AGE = 21 .OR. AGE = 26
  COUNT FOR AGE = 25 .OR. LAST = "Smith"
  COUNT FOR AGE = 26 .AND. MARRIED
  COUNT FOR AGE = 30 .AND. STATE = "FL"

  //..............................................................simple range
  COUNT FOR AGE > 22 .AND. AGE < 30

  lMachPass := .t.

next nPass

//................................wait for key, clear screen, reset cursor pos
?
Wait "Press any key to continue.                            "
Scroll(3,0,24,79)

lMachPass := .f.

For nPass := 1 to 2

  if nPass == 1
    MyAlert( "Now, we'll throw a longer, more;" +;
           "complex operation at Clipper.  ")
  else
    MyAlert( "And now Mach SIx..." )
  endif

  @ 4,5 say "   [ Pure CLIPPER ]              [ Optimized w/ Mach SIx ]" COLOR "w+/b"
  @ 5,5 say " --------------------------    ------------------------------"

  //..............................................complex expression w/ ranges
  COUNT FOR STATE = "CA" .AND. (AGE >= 20 .AND. AGE <= 30 ) .OR. ;
            STATE = "FL"              .AND. (AGE >= 20 .AND. AGE <= 30 )

  lMachPass := .t.

next nPass

//.................................................................SUM example
SELE testsix

nRow := Row()
nCol := Col()
MyAlert( "To save you further pain and agony, the next group;" +;
       "of examples will all be optimized by Mach SIx...  " )
DevPos( nRow, nCol )

? 'SUM FOR STATE = "CA" .AND. (AGE >= 20 .AND. AGE <= 30 )...'

//..........................................................SUM using Mach SIx
nStart := Seconds()
SUM Salary TO nSal for state = "CA" .and. (age >= 20 .and. age <= 30 )
nEnd:= Seconds()

//.............................................................display results
cOldColor := SetColor( "gr+/b" )
? "Time to sum = "+ AllTrim(str(nEnd - nStart)) +" second(s)."
SetColor(cOldColor)
?? " -- Sum value = "+AllTrim(str(nSal))
?

//.............................................................AVERAGE example
? 'AVERAGE FOR STATE = "CA" .AND. (AGE >= 20 .AND. AGE <= 30 )...'

//......................................................AVERAGE using Mach SIx
nStart := Seconds()
AVERAGE Salary TO nAvg FOR State = "CA" .AND. (Age >= 20 .AND. Age <= 30 )
nEnd:= seconds()

//.............................................................display results
cOldColor := SetColor( "gr+/b" )
? "Time to sum = "+ AllTrim(str(nEnd - nStart)) +" second(s)."
SetColor(cOldColor)
?? " -- Average value = "+AllTrim(str(nAvg))

?

//................................................................COPY example
? 'COPY TO TEMP1 FOR STATE = "CA" .AND. (AGE >= 25 .AND. AGE <= 60 )...'

//.........................................................COPY using Mach SIx
select testsix
GO TOP
nStart := Seconds()
COPY TO Temp1 FOR State = "CA" .AND. (Age >= 25 .AND. Age <= 60 )
nEnd:= seconds()

//.............................................................display results
cOldColor := SetColor( "gr+/b" )
? "Time to copy = "+ AllTrim(str(nEnd - nStart)) +" second(s)."
SetColor(cOldColor)
?

//...............................................................TOTAL Example
SET TAG TO last
? 'TOTAL FOR STATE = "CA" .AND. (AGE >= 20 .AND. AGE <= 30 ) ...'

//........................................................TOTAL using Mach SIx
nStart := Seconds()
TOTAL TO Temp2 ON Substr(Last,1) FOR State = "CA" .AND. ;
                                 (Age >= 20 .and. Age <= 30 )
nEnd:= seconds()

//.............................................................display results
cOldColor := SetColor( "gr+/b" )
? "Time to total = "+ AllTrim(str(nEnd - nStart)) +" second(s)."
SetColor(cOldColor)
?

//.............................wait for a key, clear screen, reposition cursor
wait

SCROLL(3,0,24,79)
@ 3,0

SET ORDER TO 0

//................................................LIST example using Mach SIx
? 'LIST FOR STATE = "CA" .AND. (AGE >= 20 .AND. AGE <= 30 ) .AND. MARRIED'
?
cOldColor := SetColor( "gr+/b" )
LIST First,Last,Age, Salary FOR State = "CA" .AND. ;
                            (Age >= 20 .AND. Age <= 30 ) .AND. Married
SetColor(cOldColor)

?
wait

SCROLL(3,0,24,79)
MyAlert( "Okay... One last bit o' pain and agony.  The dreaded;" +;
       "SET FILTER command.  First in pure Clipper, and then;" +;
       "optimized by Mach SIx.  Try a few PAGE-UPs and PAGE-;" +;
       "DOWNs in the browses...                             " )

//................................set optimization off for DBFNTX comparison
set optimize off
set filter to State = "CA" .and. ( Age >= 30 .and. Age <= 65 ) .and. Married

@4,0 say "DBFNTX:"
@5,0 say 'SET FILTER TO State = "CA" .and. (Age >= 30 .and. Age <= 65) .and. Married'

//............................................GO TOP to intialize the filter
@7,0 say "GO TOP..."
go top

@24,0 say "Press <Esc>ape to continue."
Browse( 7,0, 23,79 )

//.....................................set optimization back on for Mach SIx
set optimize on

SCROLL(3,0,24,79)
set filter to State = "CA" .and. ( Age >= 30 .and. Age <= 65 ) .and. Married

//................................................get level of optimization
nOpti := m6_IsFilter()
do case
case nOpti == OPT_NONE
   cRank := "None"
case nOpti == OPT_FULL
   cRank := "Full"
case nOpti == OPT_PARTIAL
   cRank := "Part"
endcase

@4,0 say "MachSIx:  Optimization level = " + cRank
@5,0 say 'SET FILTER TO State = "CA" .and. ( Age >= 30 .and. Age <= 65 ) .and. Married'

//............................................GO TOP to initialize the filter
@7,0 say "GO TOP..."
go top

@24,0 say "Press <Esc>ape to continue..."
Browse( 7,0, 23,79 )

CLS

Return(NIL)

//
//..TestCount()  --  Displays non-optimized COUNT using Clipper and
//                   an optimized COUNT with Mach Six for comparison
//
FUNCTION TestCount( cForexpr, bForblk )
   LOCAL nCount:=0, nStart:=0, nEnd:=0, nOpti:=0, cRank := ""
   LOCAL cOldColor := ""

   ? "COUNT FOR " + cForexpr + " ..."
   ?

   if !lMachPass

     RDDSetDefault( "DBFNTX" )
     SELE testntx
     nStart := Seconds()

     //.............................................evaluate count with dbEval()
     dbeval( {||nCount++}, bForblk )

     nEnd:= seconds()

     //............................................................print results
     @ ROW(),00 SAY "DBFNTX: "
     cOldColor := SetColor("gr+/b")
     ?? AllTrim(str(nEnd - nStart))
     SetColor(cOldColor)
     ?? " sec(s). -- "
     ?? AllTrim(str(nCount))+" rec(s)."
     nCount := 0
     ?
   else

     RDDSetDefault( RDDNAME )
     SELE testsix

     //................................................get level of optimization
     nOpti := m6_IsOptimze( cForexpr )
     do case
     case nOpti == OPT_NONE
        cRank := "None"
     case nOpti == OPT_FULL
        cRank := "Full"
     case nOpti == OPT_PARTIAL
        cRank := "Part"
     endcase

     nStart := Seconds()

     //......................................................count with Mach SIx
     nCount:=m6_countfor( cForexpr, bForblk )

     nEnd:= seconds()
     @ ROW(),37 SAY "MachSIx:" + cRank + " "
     cOldColor := SetColor("gr+/b")
     ?? AllTrim(str(nEnd - nStart))
     SetColor(cOldColor)
     ?? " sec(s). -- "
     ?? AllTrim(str(nCount))+" rec(s)."
     ?
  endif

RETURN ( NIL )

