/*
ķ
 Source file: MEMOCONV.PRG                                                
 Description: This is a simple program demonstrating how to convert       
              Clipper .DBT's to SIx Driver (SIXCDX) .FPT's or (SIXNSX)    
              .SMT's using a simple COPY TO routine.                      
 Written by : Team SuccessWare                                            
 Notice     : Copyright 1995 - SuccessWare 90, Inc.                       
                                                                          
                                                                          
  To convert from .DBTs to .FPTs using SIXCDX:                            
                                                                          
    Compile:  CLIPPER memoconv /n /w /dCDX                                
    Link   :  BLINKER FI memoconv @six3                                   
                                                                          
                                                                          
  To convert from .DBTs to .SMTs using SIXNSX:                            
                                                                          
    Compile:  CLIPPER memoconv /n /w /dNSX                                
    Link   :  BLINKER FI memoconv @six3                                   
                                                                          
                                                                          
  NOTE:  To convert from .FPT to .SMT memo files, use FPT2SMT.PRG.)       
                                                                          
Ľ
*/

#ifdef CDX
  #include "SIXCDX.CH"
  #define RDDNAME "SIXCDX"
  #define MEMOEXT ".FPT"
#else
  #include "SIXNSX.CH"
  #define RDDNAME "SIXNSX"
  #define MEMOEXT ".SMT"
#endif

FUNC MemoConv( cOldFile, cNewFile )

LOCAL aFile := {}

// Display a message
? "MEMOCONV - .DBT to " + MEMOEXT + " Memo File Converter"
?

// Make sure we got the correct parameters
IF ( PCOUNT() < 2 )
  ? "USAGE:  MEMOCONV <OldFile> <NewFile>"
  ?
  ? "where:  <OldFile> = Name of the database containing the memo"
  ? "                    field to convert."
  ? "        <NewFile> = New file to create."
  ?
  ? "   Ex:  MEMOCONV test test2"
  ?
  ? " NOTE:  Creates a new .DBF and " + MEMOEXT + " file, which must be"
  ?? " used together."
  Return(1)
ENDIF

cOldFile := Upper( AllTrim( cOldFile ))
cNewFile := Upper( AllTrim( cNewFile ))

IF "." $ cOldFile
  cOldFile := Left( cOldFile, at( ".", cOldFile )-1 )
ENDIF

IF "." $ cNewFile
  cNewFile := Left( cNewFile, at( ".", cNewFile )-1 )
ENDIF

IF cOldFile == cNewFile
  Alert( "ERROR: Source and destination;" +;
         "file names MUST be different!")
  Quit
ENDIF

// Check for the original file
IF !File( cOldFile + ".DBF" )
  ? "Cannot find " + cOldFile + "!"
  ?
  Return(2)
ENDIF

// Set the default RDD to the selected SIx Driver flavor
RDDSetDefault( RDDNAME )

// Open the original file (using SIXNTX)
USE (cOldFile) VIA "DBFNTX"

// Show that we're doing something
? "Working"

// Now copy to the new file (with nifty UDF)
COPY TO (cNewFile) FOR ShowDot()

// Look, we made it!
?? "Done!"
?

// Clean up before leaving
CLOSE ALL

// Compare file sizes
aFile := Directory( cOldFile + ".DBT" )
? PadR( "Old " + cOldFile + ".DBT Size",21 ) + ": "
?? Str( aFile[1,2] ) + " bytes"
aFile := Directory( cNewFile + MEMOEXT )
? PadR( "New " + cNewFile + MEMOEXT + " Size",21 ) + ": "
?? Str( aFile[1,2] ) + " bytes"
?

Return(0)

FUNC ShowDot()
  ?? "."
Return (.T.)
