/*
  ķ
   Program...: MEMOTEST.PRG                                              
   Author....: Team SuccessWare                                          
   Notice....: Copyright 1995 - SuccessWare 90, Inc.                     
   Purpose...: Detects, Reports, and optionally blanks out MEMO field    
               pointers which are duplicates or invalid (too high).      
                                                                         
   Compile...: CLIPPER memotest /n/w                                     
   Link......: BLINKER FI memotest, sixnsx @six3                         
  Ķ
   This MEMOTEST.EXE utility will skip through your .DBF file,           
   displaying the offset value in all MEMO fields for each record in the 
   database.  If an offset value is larger than it should be, based on   
   the size of the memo file, a warning message will be displayed to the 
   right of that entry.  Also, if an offset value is too small, where it 
   points within the memo file header (the first 512 bytes), a warning   
   message will also be displayed.                                       
                                                                         
   MEMOTEST will also alert you if duplicate memo block references       
   appear for more than one MEMO field.                                  
                                                                         
   If any discrepancies are reported, they can be cleared by rerunning   
   MEMOTEST with the /B option.  This will blank out the offending MEMO  
   field entry in the DBF file.                                          
                                                                         
   NOTE:  The DBFINFO.EXE utility, included with the SIx Driver, will    
          check for, and correct, simple database and memo file header   
          corruptions, which MEMOTEST.EXE does not do.                   
  Ľ
*/

#include "sixnsx.ch"
#include "common.ch"
#include "dbstruct.ch"
#include "simpleio.ch"
#include "fileio.ch"

REQUEST SIXCDX
REQUEST SIXNTX

FUNC Memotest( cFile, cBlank, cVerbose, cZAP )

  LOCAL nHead := 0, aStruct := {}, hDBF := 0, nStart := 0, nCount := 0
  LOCAL cOffset := Space(10), nRecord := 1, cHead := Space(1)
  LOCAL cMemoExt := "", nFileLen := 0, nBlkSize := 0, nFlds := 0
  LOCAL lBlank := .F., cMemoFld := Space(10), nOffset := 0, lVerbose := .F.
  LOCAL aOpts := {}, nRow := Maxrow(), nCol := Maxcol(), nDupes := 0
  LOCAL nInvalid := 0, lError := .F.
  LOCAL lZAP := .F., nZapped := 0, cMemoHead := Space(512)

  DEFAULT cBlank TO ""
  DEFAULT cVerbose TO ""
  DEFAULT cZAP TO ""

  aOpts := {cBlank, cVerbose, cZAP}

  CLS

  IF pCount() = 0
    ? "MEMOTEST v1.5 - Memo Field Validation Utility for .DBT/.FPT/.SMT Files"
    ? "(c)1993-95 - SuccessWare Int'l  The Data Driver Pioneers  909-699-9657"
    ?
    ? "  USAGE:  MemoTest <dbf_name>[.DBF] [/B] [/V] [/Z]"
    ?
    ? "          /B = Blank out invalid memo pointers"
    ? "          /V = Verbose Mode"
    ? "          /Z = ZAP memo file (remove all memo entries)"
    ?
    ? "  NOTE:  Output can be routed to a file by using the DOS redirection (>)"
    ? "         flag.  For example, to send the output to FOO.TXT:"
    ?
    ? "         MEMOTEST test > foo.txt"
    ?
    QUIT
  ENDIF

  // Scan command-line for switches
  lBlank := aScan( aOpts, {|c| Upper(c) == "/B"} ) > 0
  lVerbose := Ascan(aOpts, {|c| Upper(c) == "/V"}) > 0
  lZAP := aScan( aOpts, {|c| Upper(c) == "/Z"} ) > 0

  IF lBlank
    IF Alert( "The /B option will BLANK out the existing value;" +;
              "of all MEMO fields with invalid or duplicate   ;" +;
              "offset values.  Are you SURE you want to go on?", +;
              {"NO", "Yes"} ) = 1
      QUIT
    ENDIF
  ENDIF

  IF lZAP
    IF Alert( "The /Z option will BLANK out the existing value;" +;
              "of all MEMO fields.  Are you SURE you want to  ;" +;
              "go on?                                         ", +;
              {"NO", "Yes"} ) = 1
      QUIT
    ENDIF
  ENDIF

  cFile := Upper( iif( At(".", cFile) > 0, Left(cFile, At(".", cFile) - 1), cFile))

  IF .NOT. file(cFile + ".DBF")
    Alert(cFile + ".DBF not found!")
    QUIT
  ELSE
    hDBF := fOpen(cFile + ".DBF", FO_READ + FO_DENYNONE )
    IF hDBF = -1
      Alert("Unable to open " + cFile + ".DBF! : 1")
      QUIT
    ENDIF

    fRead(hDBF, @cHead, 1)
    fClose(hDBF)

    IF cHead == Chr(6)   .OR. cHead == Chr(134) .OR. ;
       cHead == Chr(230) .OR. cHead == Chr(246)
      Alert("Encrypted Table:  Not Supported!")
      QUIT
    ENDIF

    IF cHead == Chr(3)
      Alert(cFile + ".DBF has no memo file!")
      QUIT
    ENDIF

    IF cHead == Chr(131)
      cMemoExt := ".DBT"
    ELSEIF cHead == Chr(229)
      cMemoExt := ".SMT"
      cMemoFld := Replicate( Chr(0), 10 )
    ELSEIF cHead == Chr(245)
      cMemoExt := ".FPT"
    ELSE
      Alert("Invalid Database/Memo Type: " + Alltrim(Str(Asc(cHead))))
      QUIT
    ENDIF
  ENDIF

  IF file(cFile + cMemoExt)
    nFileLen := Directory( cFile + cMemoExt )[1, 2]
  ELSE
    Alert(cFile + cMemoExt + " file not found!")
    QUIT
  ENDIF

  Sx_AutoOpen(.F.)

  IF cMemoExt == ".DBT"
    nBlkSize := 512
    cMemoHead := L2Bin(1) + Replicate( Chr(0), 508 )
    USE (cFile) VIA "SIXNTX"

    IF !Used()
      Alert("Unable to open " + cFile + ".DBF : 2")
      QUIT
    ENDIF
  ELSE
    IF cMemoExt == ".FPT"
      USE (cFile) VIA "SIXCDX"
      nBlkSize := Sx_MemoBlk()
      cMemoHead := lsSwapBytes( L2Bin( 8 )) +;
                   lsSwapBytes( L2Bin( 64 ))+;
                   Replicate( Chr(0), 504 )
    ELSE
      USE (cFile) VIA "SIXNSX"
      nBlkSize := Sx_MemoBlk()
      cMemoHead := L2Bin( 8 ) + L2Bin( 64 ) + Replicate( Chr(0), 504 )
    ENDIF

    IF !Used()
      Alert("Unable to open " + cFile + ".DBF : 3")
      QUIT
    ENDIF
  ENDIF

  nHead := Header()
  aStruct := dbStruct()
  nFlds := Len(aStruct)
  dbCloseArea()

  FOR nCount := 1 TO nFlds
    IF aStruct[nCount, DBS_TYPE] == "M"
      EXIT
    ENDIF
    nStart += aStruct[nCount, DBS_LEN]
  NEXT

  hDBF := fOpen(cFile + ".DBF", FO_READWRITE + FO_DENYNONE )

  IF hDBF = -1
    Alert("Unable to open " + cFile + ".DBF! : 4")
    QUIT
  ENDIF

  dbCreate("_TEST_.$$$", {{"RECORD", "N", 10, 0}, {"OFFSET", "N", 10, 0}})

  IF !file("_TEST_.$$$")
    Alert("Error Creating Temporary File!")
    QUIT
  ENDIF

  USE _TEST_.$$$ VIA "SIXNSX"
  INDEX ON _test_->OFFSET TO _TEST_.!!!

  fSeek(hDBF, nHead + nStart + 1, 0)

  DO WHILE fRead(hDBF, @cOffset, 10) == 10

    lError := .F.
    IF cMemoExt == ".SMT"
      cOffset := Right( cOffset, 4)
      cOffset := iif( Empty( cOffset ), "", Str( Bin2L( cOffset )))
    ENDIF

    @ nRow, 0 SAY "Rec " + Padr(Alltrim(Str(nRecord)), 6)

    IF lVerbose
      DevPos( nRow, 0 )
      ?? "Rec " + Padr( Alltrim( Str( nRecord )), 6) + ": "
      ?? Padr( aStruct[nCount, DBS_NAME], 10 ) + ": "
      ?? "Block "
      ?? Padr( iif( Empty( cOffset), "<none>", Alltrim( cOffset )), 10 )
    ENDIF
    nOffset := Val(cOffset)

    IF lZAP
      fSeek(hDBF, -10, 1)
      IF fWrite(hDBF, cMemoFld, 10) = 10
        ?? " : Blanked!"
        nZapped++
      ELSE
        ?? " : Error!"
      ENDIF
    ELSEIF (nOffset * nBlkSize > nFileLen) .or. ;
           ((nOffset > 0) .and. (nOffset * nBlkSize < 512))
      ++nInvalid
      IF !lVerbose
        ?? "Rec " + Padr( Alltrim( Str( nRecord )), 6) + ": "
        ?? Padr( aStruct[nCount, DBS_NAME], 10 ) + ": "
        ?? "Block "
        ?? Padr( iif( Empty( cOffset ), "<none>", Alltrim( cOffset )), 10)
      ENDIF

      ?? " <= Invalid Block"
      Tone(1000, 1)

      IF lBlank
        fSeek(hDBF, -10, 1)
        IF fWrite(hDBF, cMemoFld, 10) = 10
          ?? " : Fixed!"
        ELSE
          ?? " : Error!"
        ENDIF
      ENDIF
      lError := .T.
    ELSEIF dbSeek(nOffset)
      ++nDupes
      IF !lVerbose
        ?? "Rec " + Padr(Alltrim(Str(nRecord)), 6) + ": "
        ?? Padr(aStruct[nCount, DBS_NAME], 10) + ": "
        ?? "Block "
        ?? Padr(IF(Empty(cOffset), "<none>", Alltrim(cOffset)), 10)
      ENDIF

      ?? " <= Dupe w/ Rec " + Alltrim(Str(_Test_->RECORD))
      Tone(1000, 1)

      IF lBlank
        fSeek(hDBF, -10, 1)
        IF fWrite(hDBF, cMemoFld, 10) = 10
          ?? " : Fixed!"
        ELSE
          ?? " : Error!"
        ENDIF
      ENDIF
      lError := .T.
    ELSEIF nOffset > 0
      dbAppend()
      _Test_->RECORD := nRecord
      _Test_->OFFSET := nOffset
    ENDIF

    IF nFlds > 1
      IF nCount >= nFlds
        fSeek(hDBF, 1, 1)
        nCount := 0
        ++nRecord
      ENDIF

      DO WHILE aStruct[nCount + 1, DBS_TYPE] <> "M"
        fSeek(hDBF, aStruct[++nCount, DBS_LEN], 1)
        IF nCount >= nFlds
          fSeek(hDBF, 1, 1)
          nCount := 0
          ++nRecord
        ENDIF
      ENDDO
      ++nCount
    ELSE
      fSeek(hDBF, 1, 1)
      ++nRecord
    ENDIF

    IF lVerbose .OR. lError
      ?
    ENDIF
  ENDDO

  CLOSE ALL
  fClose( hDBF )

  ?
  ? "Status report for " + cFile + ".DBF and " + cFile + cMemoExt + ":"
  ? "----------------------------------------------"
  ?
  ? "Records Processed: " + Alltrim( Str( nRecord - 1 ))
  IF lZAP
    ? "MEMOs Blanked    : " + Alltrim( Str( nZapped ))
    hDBF := fCreate( cFile + cMemoExt )
    fWrite( hDBF, cMemoHead, 512 )
    fClose( hDBF )
  ELSE
    ? "Invalid Offsets  : " + Alltrim( Str( nInvalid ))
    ? "Duplicate Offsets: " + Alltrim( Str( nDupes ))
  ENDIF
  ?

Return( NIL )

//------------------------------------------------------------------//

FUNC Sx_DefTrigger( nEvent, nArea, nFieldPos, xTrigVal )

  DO CASE

    // Clean up temp files
    CASE nEvent == EVENT_POSTCLOSE
      fErase( "_TEST_.$$$" )
      fErase( "_TEST_.!!!" )

  ENDCASE

Return( .T. )

//------------------------------------------------------------------//

FUNC lsSwapBytes( cVal )

  LOCAL i := 0, cNewVal := ""

  FOR i := Len(cVal) TO 1 STEP -1
    cNewVal += SubStr( cVal, i, 1 )
  NEXT

RETURN( cNewVal )


