/*
  ķ
                                                                           
   Program...: MULTIKEY.PRG                                                
   Purpose...: Demonstrates adding keywords from a memo field or from a    
               substring within a GET into a multi-keyed RYO index.        
   Notice....: Copyright 1993-95 - SuccessWare 90, Inc.                    
   Author....: Team SuccessWare                                            
                                                                           
   Compile...: CLIPPER multikey /n/w /dCDX  (for SIXCDX)                   
           or: CLIPPER multikey /n/w /dNSX  (for SIXNSX)                   
                                                                           
   Link......: BLINKER FI multikey, sixtools, sixcdx @six3  (for SIXCDX)   
           or: BLINKER FI multikey, sixtools, sixnsx @six3  (for SIXNSX)   
                                                                           
   NOTES:  This program will create its own database and index file if     
           they don't exist already.  Add text to the GET's and MEMO       
           fields.   Then add keywords for doing searches by position-     
           ing the cursor on the word to be added and pressing the <F4>    
           key.                                                            
                                                                           
           Be sure that you have saved any new text edits by either        
           pressing <Enter> in the GET, or by pressing the <F2> key in     
           the memo field before attempting to add a new keyword.          
                                                                           
           The NAME field uses a UDF in the VALID clause of the GET to     
           automatically maintain an index key for each word in the field. 
                                                                           
           All signifigant words in the memo field will also be automatic- 
           ally be added as keys into the index.                           
                                                                           
  Ľ
*/

#ifdef CDX
  #include "SIXCDX.CH"
  #define RDDNAME "SIXCDX"
  #define INDEXEXT ".CDX"
#else
  #include "SIXNSX.CH"
  #define RDDNAME "SIXNSX"
  #define INDEXEXT ".NSX"
#endif

#include "inkey.ch"
#include "memoedit.ch"

#define TRUE  .T.
#define FALSE .F.

#xtranslate String( <cStr> )  =>  AllTrim( Str( <cStr> ))

STATIC nRecNo := 1, lStop := FALSE

EXTE sxChar

/*-------------------------( MemoKey Function )-------------------------*/

FUNC MultiKey()

LOCAL aFields := {}, nCount, nKey, cOldColor := SetColor("bg+/b")
LOCAL nCurs := SetCursor(1)
MEMVAR GetList
FIELD NOTES

SET SCOREBOARD off

// Create MULTI.DBF if it doesn't exist; Open MULTI.DBF
RDDSetDefault( RDDNAME )
if !file("MULTI.DBF")
  CLS
  ? "Creating Data and Index Files..."
  aFields := {{"NAME", "C",40,0},;
              {"ADDR", "C",40,0},;
              {"CITY", "C",14,0},;
              {"STATE","C", 2,0},;
              {"ZIP",  "C",10,0},;
              {"PHONE","C",13,0},;
              {"NOTES","M",10,0}}
  dbCreate( "MULTI", aFields )

  // Make sure an old MULTI.CDX file isn't left out there...
  if file("MULTI" + INDEXEXT)
    fErase("MULTI" + INDEXEXT)
  end
endif

// Open MULTI.DBF
use MULTI excl

// Append a blank record if DBF is empty
if LastRec() == 0
  dbAppend()
endif

// Create MULTI.CDX if it doesn't exist; Select KEYWORD tag
if !file("MULTI" + INDEXEXT)
  index on sxChar(15) tag KEYWORD
endif
set tag to
dbGoTop()

// Set up screen and Display single-record field data
Scroll()
@ 6,58 to 21,77 double
@ 6,59 say " Key Words:    "
@ 0, 1 say " Multi-Keyed RYO Index Demo " color "b/w"

SetColor("gr+/n")
@ 1,31 to 4,77
@ 2,32 say " All significant keywords will automatically "
@ 3,32 say " be maintained for the NAME and MEMO fields. "

SetColor("w+/b,b/w,,,b/w")
@ 2, 0 say "Keywords: " color "bg+/b"
@ 3, 0 say "Record  : " color "bg+/b"
@ 4, 0 say "Search  : " color "bg+/b"
@ 6, 1 to 13,56
@15, 1 to 21,56
@15, 3 say " Memo "

// Set up hot-keys
HotKeys()
CLEAR TYPEAHEAD
SetColor("w+/n,b/w,,,bg+/n")
@ 7, 2 clear to 12,55

do while TRUE

  @ 3,10 say padr( String( RecNo()) + "/" + String( lastrec()), 10 ) ;
             color "w+/b"

  // Display memo field contents in Notes window
  MemoEdit( NOTES,16,2,20,55,FALSE,FALSE)

  // Display keywords for this record
  KeyDisp()

  // Display Record Data
  @ 8, 3 say "Name.....:" get NAME  VALID FieldKey()
  @ 9, 3 say "Address..:" get ADDR
  @10, 3 say "City.....:" get CITY
  @10,29 say "State:"     get STATE picture "@!"
  @10,39 say "Zip:"       get ZIP   picture "99999-9999"
  @11, 3 say "Phone....:" get PHONE picture "(999)999-9999"
  read

  nKey := LastKey()
  if nKey == K_ESC
    exit
  elseif nKey == K_PGUP
    PrevRec()
  elseif nKey == K_PGDN
    NextRec()
  endif

enddo

CLOSE ALL
SetColor( cOldColor )
SetCursor( nCurs )
KillKeys()
CLS

Return(NIL)

/*-------------------------( HotKeys Function )-------------------------*/

FUNC HotKeys()

SetKey( K_F2, {|| __KeyBoard( K_CTRL_W )} )
SetKey( K_F3, {|| KeySearch()} )
SetKey( K_F4, {|| AddKey()} )
SetKey( K_F5, {|| KeyDel()} )
SetKey( K_F8, {|| EditMemo()} )

@23, 2 say " <F2> - Save     " color "b/w"
@24, 4 say " <F3> - Search   " color "b/w"
@23,21 say " <PgUp> - Prev   " color "b/w"
@24,23 say " <PgDn> - Next   " color "b/w"
@23,40 say " <F4> - Add Key  " color "b/w"
@24,42 say " <F5> - Del Key  " color "b/w"
@23,59 say " <F8> - Edit Memo" color "b/w"
@24,61 say " <Esc>- QUIT     " color "b/w"

Return(NIL)

/*------------------------( KillKeys Function )-------------------------*/

FUNC KillKeys()

SetKey( K_F2, NIL )
SetKey( K_F3, NIL )
SetKey( K_F4, NIL )
SetKey( K_F5, NIL )
SetKey( K_F8, NIL )


Return(NIL)

/*------------------------( EditMemo Function )-------------------------*/

FUNC EditMemo()

LOCAL nOldRec := RecNo()
LOCAL cOldVal
FIELD NOTES

// Clear hot-keys
SetKey( K_F2, NIL )
SetKey( K_F3, NIL )
SetKey( K_F4, NIL )
SetKey( K_F5, NIL )
SetKey( K_F8, NIL )

@23,21 say " <PgUp> - Prev   " color "n+/w"
@24,23 say " <PgDn> - Next   " color "n+/w"
@23,59 say " <F8> - Edit Memo" color "n+/w"
@24,61 say " <Esc>- RETURN   " color "b/w"

lStop := FALSE
cOldVal := multi->NOTES
do while !lStop
  multi->NOTES := MemoEdit( NOTES,16,2,20,55,TRUE,"MemoUDF" )
  dbGoto( nOldRec )
enddo
CLEAR TYPEAHEAD

// Update keywords in memo
AutoKey( cOldVal, multi->NOTES )

// Reset hot-keys
HotKeys()

Return(NIL)

/*-------------------------( KeyDisp Function )-------------------------*/

FUNC KeyDisp()

LOCAL aKeys[14], nKeyNo := 1, nCount, cOldColor := SetColor(), nKeys := 1

afill( aKeys, "" )

SET TAG TO keyword
if Sx_FindRec( nRecNo, FALSE )
  aKeys[1] := Sx_KeyData()
  do while Sx_FindRec( nRecNo, TRUE )
    if nKeyNo < 14
      aKeys[ ++nKeyNo ] := Sx_KeyData()
    endif
    ++nKeys
  enddo
else
  nKeys := 0
endif
@ 6,72 say Str( nKeys, 2 ) color "w+/b"
@ 2,10 say padr( String( Sx_KeyCount()), 2 ) color "w+/b"
SET TAG TO

SetColor( "g+/n" )
@ 7,59 clear to 20,76
for nCount := 1 to nKeyNo
  @ 6 + nCount, 60 say aKeys[ nCount ]
next

SetColor( cOldColor )
dbGoto( nRecNo )

Return(NIL)

/*-------------------------( FieldKey Function )------------------------*/

FUNC FieldKey()

LOCAL oCurGet := GetActive()

Return( AutoKey( oCurGet:original, oCurGet:buffer ))

/*--------------------------( AutoKey Function )------------------------*/

FUNC AutoKey( cOld, cNew )

LOCAL cTempStr, cWord, nRecNo := RecNo()

// if field hasn't changed, return now
if cOld == cNew
  Return(TRUE)
endif

SET TAG TO keyword

// Loop through old field buffer, deleting old keys
cTempStr := Upper( cOld )                                 // Store old value
do while !empty( cTempStr )
  cWord := Left( cTempStr, at( " ", cTempStr + " " )-1 )  // Get next word
  cTempStr := AllTrim( SubStr( cTempStr, Len( cWord )+2)) // Reset cTempStr

  // Clean word of any leading or trailing punctuation marks
  cWord := CleanWord( cWord )

  if SeekRec( cWord, nRecNo )                             // SEEK for cWord
    Sx_KeyDrop()                                          // Delete key
  else
    dbGoto( nRecNo )
  endif
enddo

// Add new keys to index
cTempStr := Upper( cNew )                                 // Store new value
do while !empty( cTempStr )
  cWord := Left( cTempStr, at( " ", cTempStr + " " )-1 )  // Get next word
  cTempStr := AllTrim( SubStr( cTempStr, Len( cWord )+2)) // Reset cTempStr

  // Clean word of any leading or trailing punctuation marks
  cWord := CleanWord( cWord )

  // Ignore selected words ("A", "I", "THE", etc...)
  IF cWord $ "A I THE OF IN AN AND FOR IS IT INC CO LTD"
    LOOP
  ENDIF

  Sx_KeyAdd( "KEYWORD", 1, sxChar( 15, cWord ))           // Add key
enddo

KeyDisp()
SET TAG TO

Return(TRUE)

/*-------------------------( SeekRec Function )-------------------------*/

FUNC SeekRec( cVal, nRec )

if dbSeek( cVal )
  do while cVal == AllTrim( Sx_KeyData() )
    if nRec == RecNo()
      Return(TRUE)
    endif
    dbSkip()
  enddo
endif

Return(FALSE)

/*--------------------------( KeyDel Function )-------------------------*/

func KeyDel()

LOCAL nKeyPos, aKeys := {}, aKeyNums[1], nChoice
LOCAL cOldColor := SetColor( "g+/n,b/w,,,g+/n" )

SET TAG TO keyword
if Sx_FindRec( nRecNo, FALSE )
  aadd( aKeys, " " + Sx_KeyData())
  do while Sx_FindRec( nRecNo, TRUE )
    AAdd( aKeys, " " + Sx_KeyData())
  end
else
  SET TAG TO
  Return(NIL)
endif

@ 7,59 clear to 20,75
nKeyPos := aChoice( 7,59,20,75,aKeys )

if nKeyPos # 0
  if dbSeek( AllTrim( aKeys[ nKeyPos ] ))
    nChoice := MyAlert( "Delete keyword?: " + Trim( Sx_KeyData()), {"Yes", "No"} )
    if nChoice == 1
      Sx_KeyDrop()
    endif
  endif
endif

KeyDisp()
SET TAG TO
SetColor( cOldColor )

Return(NIL)

/*-------------------------( AddKey Function )--------------------------*/

FUNC AddKey()

LOCAL oCurGet := GetActive(), cTempStr, nByte

// Save everything in field up to this byte to temporary string
cTempStr := substr( oCurGet:original, 1, Col() - oCurGet:col + 1 )

// Check if cursor was on a space
if right( cTempStr, 1 ) == " "
  Return(NIL)
endif

// Locate last space in the temporary string; Gets first byte of keyword
nByte := rat( " ", cTempStr ) + 1

// Save rest of field to temp string; add a space at the end
cTempStr := substr( oCurGet:original, nByte ) + " "

// Add Key To Index
SaveKey( cTempStr )

Return(NIL)

/*-------------------------( MemoUDF Function )-------------------------*/

FUNC MemoUDF( nMode, nRow, nCol )

LOCAL cKey, nByte, cScreen, cTempStr
FIELD NOTES

if nMode == ME_UNKEY .or. nMode == ME_UNKEYX


  cKey := LastKey()

  do case

    case cKey == K_ESC
      lStop := TRUE

    case cKey == K_F2
      Return( K_CTRL_W )

    case cKey == K_F3
      if KeySearch()
        lStop := TRUE
        Return( K_CTRL_W )
      endif

    case cKey == K_PGUP
      PrevRec()
      lStop := TRUE
      Return( K_CTRL_W )

    case cKey == K_PGDN
      NextRec()
      lStop := TRUE
      Return( K_CTRL_W )

    case cKey == K_F4

      // Get current byte in memo field
      nByte := mlctopos( NOTES, 70, nRow, nCol )

      // Save everything in memo field up to this byte to temporary string
      cTempStr := substr( NOTES, 1, nByte )

      // Check if cursor was on a space or carriage Return(EOL)
      if right( cTempStr, 1 ) $ chr(32)+chr(13)
        Return(ME_DEFAULT)
      endif

      // Convert carriage returns to spaces
      cTempStr := StrTran( cTempStr, chr(13), chr(32))

      // Locate last space in the temporary string; Gets first byte of keyword
      nByte := rat( " ", cTempStr ) + 1

      // Save rest of memo field to temp string; add a space at the end
      cTempStr := substr( NOTES, nByte ) + " "

      // Add Key To Index
      SaveKey( cTempStr )

    case cKey == K_F5
      KeyDel()

  endcase
endif

Return(ME_DEFAULT)

/*-------------------------( PrevRec Function )-------------------------*/

FUNC PrevRec()

nRecNo--
dbGoto( nRecNo )
if bof()
  tone(200,1)
  nRecNo++
  dbGoto( nRecNo )
endif
KeyDisp()

Return(K_PGUP)

/*-------------------------( NextRec Function )-------------------------*/

FUNC NextRec()

LOCAL cName := multi->NAME

dbGoto( ++nRecNo )

// If at EOF and previous record was not empty, add a new record
if eof()
  if !empty( cName )
    dbAppend()
  else
    dbGoto( --nRecNo )
  endif
endif
KeyDisp()

Return(K_PGDN)

/*------------------------( KeySearch Function )------------------------*/

FUNC KeySearch()

LOCAL cSearchStr := Space(15), nOldRec
LOCAL cOldColor := SetColor("bg+/n,w+/b")
LOCAL GetList := {}

@ 4, 10 say Space(15)
@ 4, 10 get cSearchStr picture "@!"
read
nOldRec := RecNo()
if LastKey() # K_ESC
  SET TAG TO keyword
  if dbSeek( trim( cSearchStr ))
    nRecNo := RecNo()
    if !rlock()
      MyAlert( "Record In Use!" )
    endif
    CLEAR GETS
  else
    MyAlert( "Not Found!" )
    dbGoto( nOldRec )
  endif
  SET TAG TO
endif
SetColor( cOldColor )

Return( Found() )

/*-------------------------( SaveKey Function )-------------------------*/

FUNC SaveKey( cTempStr )

LOCAL cWord

// Extract entire key word (should be the first word in cTempStr)
cWord := upper( left( cTempStr, at( " ", cTempStr ) - 1 ))

// Clean word of any leading or trailing punctuation marks
cWord := CleanWord( cWord )

// If keyword is not an empty string, add it to KEYWORD tag
if !empty( cWord )
  SET TAG TO keyword

  // Make sure cWord is 15 characters or less; trim off extra spaces
  cWord := trim( padr( cWord, 15 ))

  if Sx_KeyAdd( "KEYWORD", 1, sxChar( 15, cWord ))
     MyAlert( "Keyword " + cWord + " added!" )
     KeyDisp()
  else
     MyAlert( "Keyword " + cWord + " NOT added!" )
  endif
  SET TAG TO
endif

Return(NIL)

/*------------------------( CleanWord Function )------------------------*/

FUNC CleanWord( cWord )

// Strip off any leading quotes, parens, brackets, or curly braces
do while ( !isalpha( left( cWord, 1 )) .AND. !isdigit( left( cWord, 1 )))
  cWord := substr( cWord, 2 )
  if empty( cWord )
    exit
  endif
enddo

// Strip off any ending punctuation marks (,.!?) or quotes and parens
do while ( !isalpha( right( cWord, 1 )) .AND. !isdigit( right( cWord, 1 )))
  cWord := left( cWord, len( cWord ) - 1 )
  if empty( cWord )
    exit
  endif
enddo

Return( cWord )
