/*
ķ
 Source file: OBSOLETE.PRG                                                
 Description: Compatibility Functions for SIx 3.0                         
 Written by : Team SuccessWare                                            
 Notice     : Copyright 1995 - SuccessWare 90, Inc.                       
                                                                          
  NOTES:  File-based Order Management functions are considered obsolete   
  in SIx 3.0. They are more confusing than helpful given the sounder      
  Order List Management model embraced in CA-Clipper 5.2.                 
  Here is CA-Clipper source code that will perform the same tasks as      
  the former SIx Driver library functions.                                
                                                                          
  Also note that the OBSOLETE.CH file contains SIx driver functions that  
  are directly replaced with corresponding functions in CA-Clipper 5.2    
                                                                          
Ľ
*/
#define ORDER_LIMIT 256
#define BAG_LIMIT    50

/*
  Syntax:  Sx_FileOrder()
  Returns: Position of current index file in the index file list
  Works like indexord() in Clipper 5.01a
*/
Function Sx_FileOrder()
   local nSaveOrder  := indexord()  ,;
         nFileOrder  := 0           ,;
         nextOrder   := 1           ,;
         nTotalTags  := 0           ,;
         nTagCount
   do while nextOrder <= nSaveOrder
      if (nTagCount := Sx_tagCount( ordbagName(nextOrder) )) > 0
         nTotalTags += nTagCount
         nextOrder := nTotalTags + 1
         nFileOrder++
      else
         exit
      endif
   enddo
Return nFileOrder
/*
  Syntax:  Sx_SetFileOrd( <nFileOrder> )
           <nFileOrder> is the position of an index file in
                        the index file list.
  Returns:  The previously set file order position.
  Selects 1st tag in an index file given the file order position in
  the SET INDEX TO list. Works like dbSetOrder() under Clipper 5.01a.
*/
Function Sx_SetFileOrd( nTarget )
   local nSaveOrder  := indexord()     ,;
         nOldFileOrd := Sx_FileOrder() ,;
         nextOrder   := 1              ,;
         nTotalTags  := 0              ,;
         nTagCount, i
   if !(nTarget == NIL)
      for i := 1 to nTarget
         ordSetFocus( nextOrder )
         if (nTagCount := Sx_tagCount()) > 0
            nTotalTags += nTagCount
            nextOrder := nTotalTags + 1
         else
            ordsetfocus(nSaveOrder)
            exit
         endif
      next
   endif
Return nOldFileOrd

/*
  Syntax:   Sx_SetTag([cTag | nOrder] [,cBag])
            cTag     = Name of the tag to make active
            nOrder   = Position of order in the order list
            cBag     = Name of the file that contains the tag or order
  Returns:  <logical> .T. if order changed successfully.
            otherwise returns .F. and former order is restored.
  3/3/95
  ------
  Fix bug that returns .t. when called like sx_settag(x,y) and there are no tags open.
  Test for 0 tagcount is tagEmpty(sx_indexName(1)). Don't use sx_tagCount() because 
  if we're at order 0 now, tagcount() w/ no params returns 0, even if an index file is open.
     
  3/8/95   
  ------
  Special Cases:
      if no dbf open      --> return false
      if xTag == NIL      --> return false
      if no ix open       --> return empty(xTag) 
*/
Function Sx_SetTag(xTag, cBag)
   local cTag, nCount, oldOrd

   local lRet        := .t. 
   local nTagCount   := sx_tagCount()

   if empty(alias()) .or. xTag == NIL
      lRet := .f.

   elseif  empty(sx_indexName(1))
      // if we say sx_settag(0) or sx_setTag("") then
      // we return true even if there is no index open.

      lRet := empty( xTag )
      
   else
      
      oldOrd := ordNumber()
      if empty(cBag)
         if empty(ordBagName()) .and. !empty( xTag )
            ordSetFocus(xTag)
            lRet := !empty( ordBagName() )
         else
            ordSetFocus(xTag)
            lRet := (ordBagName(xTag) == ordBagName())
         endif
      else
         if valtype(xTag) == "N"
            nTagCount := sx_tagCount(cBag)
            // if passed number is greater than number of tags
            // in the passed orderbag, return false
            if xTag > nTagCount .or. xTag < 0
               lRet := .f.
            else
               // we have to do this because, although clipper's ordsetfocus()
               // works correctly with a params <cTag>,<cBag> it does
               // NOT work correctly with <nTag>, <cBag>. It ignores the
               // orderbagname if a number is passed and just sets
               // order to the orderlist number represented by <nTag>.
               nCount := bag2Tag( cBag )
               ordSetFocus( nCount + xTag - 1)
               lRet := (ordBagName() == ordBagName( nCount + xTag - 1))
            endif
         else
            ordSetFocus(xTag, cBag)
            if valtype(xTag) == "N"
               lRet := ( ordName() == ordName( xTag, cBag ))
            else
               lRet := (ordNumber() == ordNumber(xTag, cBag ))
            endif
         endif
      endif
      if !lRet
          ordSetFocus(oldOrd)
      endif
   endif
return lRet

/*
   Syntax:  Sx_TagCount([xBag])
            xBag = Bag Name or Bag Position in list of Index Files
  Returns:  <numeric> Number of tags in the current or specified
            index (order bag). 0 if the order is invalid, no index is open,
            or if the current order is 0 and no parameters were passed.
*/
Function Sx_TagCount(xBag)
   local nOrd, cBagName
   local nCnt := 0
   // get order list position of first tag in the passed bag.
   if !empty(alias())
      nOrd := bag2Tag(xBag)
      cBagName := ordBagName(nOrd)
      if nOrd > 0
         while cBagName == ordBagName(nOrd)
            nCnt++
            nOrd++
         enddo
      endif
   endif
Return nCnt

/*
   Return array of Tagnames in a given orderBag
   Syntax:         Sx_Tags( [cBagName | nBagPosition] )
                   cBagName = name of index file (order bag)
                   nBagPosition = position of index in list of indexes
  Returns:   <array> Array of tag names contained in the index file.
              {} if the order is invalid or no index open.
*/
Function  Sx_Tags(xBag)
   local n := 1, ar_ := {}, nCount, i, nFirstTag
   if !empty( alias() )
      nFirstTag := bag2Tag( xBag )
      nCount := Sx_tagCount(xBag)
      ar_ := array(nCount)
      for i := 1 to nCount
           ar_[i] := ordName( nFirstTag + i - 1 )
      next
   endif
Return ar_

/*
  Return orderlist position of the first order of an orderbag.
  xBag can be nBagPos, cBagName, or NIL
*/
static Function bag2Tag(xBag)
   local nOrdCnt, cBagName, nAtDot, cExt, cPrevBag
   local nBagCnt     := 1

   if xBag == NIL
      if empty( cBagName := ordBagName() )
         nOrdCnt := 0
      else
         nOrdCnt := 1
         while !( cBagName == ordBagName(nOrdCnt) )
            nOrdCnt++
         enddo
      endif
   elseif valtype(xBag) == "N"
      if empty(ordBagName(xBag))
         nOrdCnt := 0
      else
         nOrdCnt := 1
         while nBagCnt < xBag
            cBagName := ordBagName(nOrdCnt)
            nOrdCnt++
            while cBagName == ordBagName(nOrdCnt)
               nOrdCnt++
            enddo
            nBagCnt++
         enddo
      endif
   else     // xbag is a filename
      nAtDot := at(".", xBag)
      cExt   := ordBagExt()
      xBag   := upper(xBag)
      if empty( ordBagName(1) )
         nOrdCnt := 0
      else
         nOrdCnt := 1
         if nAtDot == 0 .and. cExt == ".IDX"
            while !( xBag + ".CDX" == ordBagName(nOrdCnt) )
               if  ( xBag + ".IDX" == ordBagName(nOrdCnt) )
                  exit
               endif
               nOrdCnt++
               if nOrdCnt > ORDER_LIMIT   // no xBag in the index file list
                  nOrdCnt := 0
                  exit
               endif
            enddo
         else
            if nAtDot == 0
               xBag := xBag + cExt
            endif
            while !( xBag == ordBagName( nOrdCnt ) )
               nOrdCnt++
               if nOrdCnt > ORDER_LIMIT   // no xBag in the index file list
                  nOrdCnt := 0
                  exit
               endif
            enddo
         endif
      endif
   endif
Return nOrdCnt

