/*
  ķ
                                                                          
   Program...: ROLLBACK.PRG                                               
   Author....: Team SuccessWare                                           
   Notice....: Copyright 1993-95 - SuccessWare 90, Inc.                   
   Purpose...: This is a simple program demonstrating the use of          
               Sx_RollBack() in a simple record edit.  The actual edit    
               is done in the EditRec() function below.                   
                                                                          
   Compile...: CLIPPER rollback /n/w /dCDX   (For SIXCDX)                 
           or: CLIPPER rollback /n/w /dNSX   (For SIXNSX)                 
   Link......: BLINKER FI rollback @six3                                  
                                                                          
   NOTES:  This program requires a TEST.DBF file of any size, which must  
           be created with our DBCRE8.EXE utility.  For example, to       
           create a TEST.DBF file with 1000 records, you would type this  
           at the DOS command-line:                                       
                                                                          
               C:\> DBCRE8 1000                                           
                                                                          
  Ľ
*/

#ifdef CDX
  #include "SIXCDX.CH"
  #define RDDNAME "SIXCDX"
#else
  #include "SIXNSX.CH"
  #define RDDNAME "SIXNSX"
#endif

#include "INKEY.CH"
#include "BOX.CH"


//--------------------------------------------------------------------//
FUNCTION RollBack()

LOCAL nKey := 0, nCurs := SetCursor(0)
LOCAL lBlink := SetBlink(.F.)
LOCAL cOldColor := SetColor("w+/b")

// We don't need to keep score!
SET SCOREBOARD OFF

// Open the database VIA SIx Driver RDD
USE TEST VIA RDDNAME

// Clear the screen
CLS
SetColor("r/w*")

// Draw a box
@ 5, 7, 19, 73 BOX B_DOUBLE + " "

// Draw a title
@ 5,31 SAY " ROLLBACK EXAMPLE "

// Set up a record display
@  7,10 SAY "   First:"
@  8,10 SAY "    Last:"
@  9,10 SAY "  Street:"
@ 10,10 SAY "    City:"
@ 11,10 SAY "   State:"
@ 12,10 SAY "     Zip:"
@ 13,10 SAY "HireDate:"
@ 14,10 SAY " Married:"
@ 15,10 SAY "     Age:"
@ 16,10 SAY "  Salary:"
@ 17,10 SAY "   Notes:"

// Add a help line
@ 19,21 SAY " [ENTER] = Edit record   [ESC] = Quit "

// Loop until Esc is pressed
SetColor("b/w*,w+/b,,,bg+/b")
DO WHILE (nKey != K_ESC)

  // Show the fields
  @  7,20 SAY test->FIRST
  @  8,20 SAY test->LAST
  @  9,20 SAY test->STREET
  @ 10,20 SAY test->CITY
  @ 11,20 SAY test->STATE
  @ 12,20 SAY test->ZIP
  @ 13,20 SAY test->HIREDATE
  @ 14,20 SAY test->MARRIED
  @ 15,20 SAY test->AGE    PICTURE "99"
  @ 16,20 SAY test->SALARY PICTURE "$999,999"
  @ 17,20 SAY test->NOTES  PICTURE "@S50"

  // Get a keystroke
  nKey := Inkey(0)

  DO CASE

    // Basic keystroke handler
    CASE (nKey == K_PGUP)            // Previous record
      IF !Bof()
        dbSkip(-1)
      ENDIF

    CASE (nKey == K_PGDN)            // Next record
      IF !Eof()
        dbSkip()
      ENDIF

    CASE (nKey == K_HOME)            // First record
      dbGoTop()

    CASE (nKey == K_END)             // Last Record
      dbGoBottom()

    CASE (nKey == K_ENTER)           // Edit the record
      EditRec()                      // Call our edit function

  ENDCASE

ENDDO

CLOSE ALL
SetColor( cOldColor )
SetBlink( lBlink )
SetCursor( nCurs )
CLS

Return(NIL)


//--------------------------------------------------------------------//
/*
    EditRec() - This is the edit routine.  If the user presses Esc during
                the edit then a record rollback is done to restore the
                original record.
*/

FUNC EditRec()

LOCAL nError
MEMVAR GetList

// Change the help line
@ 19,21 SAY " Press [ESC] to cancel edit "

// Edit the fields directly
@  7,20 GET test->FIRST
@  8,20 GET test->LAST
@  9,20 GET test->STREET
@ 10,20 GET test->CITY
@ 11,20 GET test->STATE
@ 12,20 GET test->ZIP
@ 13,20 GET test->HIREDATE
@ 14,20 GET test->MARRIED
@ 15,20 GET test->AGE    PICTURE "99"
@ 16,20 GET test->SALARY PICTURE "$999,999"
@ 17,20 GET test->NOTES  PICTURE "@S50"

// Turn on the cursor
SetCursor(1)

READ

// Turn off the cursor
SetCursor(0)

// If escape was pressed then rollback the record to original
IF (LastKey() == K_ESC)
  // Try to rollback record
  IF (! Sx_RollBack())
    // Oh no, there's a problem!  Tell the user.
    nError := Sx_Error()
    Alert("Unable to rollback record!  Error code: " +;
            LTrim(STR(nError)))
  ENDIF
ENDIF

// Restore original help line
@ 19,21 SAY " [ENTER] = Edit record   [ESC] = Quit "

Return(NIL)


