/*
  ķ
                                                                         
   Program...: RYODEMO.PRG                                               
   Author....: Team SuccessWare                                          
   Notice....: Copyright 1993-95 - SuccessWare 90, Inc.                  
   Purpose...: Demonstrates using the SIx Driver's Roll-Your-Own (RYO)   
               indexes as a replacement for using a logical field in a   
               DBF to maintain records "tagged" in a browse window.      
                                                                         
   Compile...: CLIPPER ryodemo /n/w /dCDX   (for SIXCDX)                 
           or: CLIPPER ryodemo /n/w /dNSX   (for SIXNSX)                 
   Compile...: CLIPPER sixtools /n/w                                     
   Link......: BLINKER FI ryodemo, sixtools @six3                        
                                                                         
   NOTES:  This program requires a TEST.DBF file of any size, which must 
           be created with our DBCRE8.EXE utility.  For example, to      
           create a TEST.DBF file with 1000 records, you would type this 
           at the DOS command-line:                                      
                                                                         
               C:\> DBCRE8 1000                                          
                                                                         
  Ľ
*/

#ifdef CDX
  #include "SIXCDX.CH"
  #define RDDNAME "SIXCDX"
  #define INDEXEXT ".CDX"
#else
  #include "SIXNSX.CH"
  #define RDDNAME "SIXNSX"
  #define INDEXEXT ".NSX"
#endif

#include "inkey.ch"

#define TRUE  .T.
#define FALSE .F.

#define UL_ROW  2
#define UL_COL  3
#define LR_ROW  MaxRow()-6
#define LR_COL  MaxCol()-3

//--------------------------------//
FUNCTION RYODemo()

LOCAL oBrow, lBrowse, nKey, nX, xOldKey, xNewKey, lBlink
LOCAL cColor, nCol, oGet, nMaxRow := MaxRow()
LOCAL nRec, nCurs := SetCursor(0), lKeyWaiting := .F.

FIELD LAST
MEMVAR GetList

// Make sure TEST.DBF exists
IF !file("TEST.DBF")
  Alert( "TEST.DBF not found!    Use the DBCRE8.EXE;" +;
         "utility to create one before running this;" +;
         "program.  For example: DBCRE8 1000       " )
  QUIT
ENDIF

SET SCOREBOARD off

// Save Blink bit status and Colors
lBlink := SetBlink(FALSE)
cColor := SetColor("w+/b")
CLS

// Delete structural index file, if it exists
CLOSE ALL
IF file( "TEST" + INDEXEXT )
  fErase( "TEST" + INDEXEXT )
ENDIF

// Open database using the SIx Driver
USE TEST VIA RDDNAME

// Create LAST tag
INDEX ON last TAG last OPTION Odometer() STEP 10

// Create TEMP RYO index
INDEX ON last TO temp EMPTY
SET TAG TO last
GO TOP

// Make shadow
SetColor("w/n")
@ UL_ROW+1, UL_COL+1 CLEAR TO LR_ROW+1, LR_COL+1

// Create a TBrowse database object
oBrow := TBrowseDB( UL_ROW, UL_COL, LR_ROW, LR_COL )
oBrow:colorSpec := "N/W*, N/BG*,, B/BG*, B/W*, B/BG*, R/W*, B/R*"
SetColor( oBrow:colorSpec )

// Change the heading separator
oBrow:headSep := " "

// Add one column for each field
oBrow:addColumn(TBColumnNew("", {|| IsTagged( oBrow )} ))
FOR nX := 1 TO FCount()
    oBrow:addColumn(TBColumnNew(FieldName(nX), FieldBlock(FieldName(nX))))
NEXT

// Draw the title
@ 0,24 SAY " Roll-Your-Own (RYO) Index Demo "
@ 0,56 SAY "" color "n/b"
@ 1,25 SAY "" color "n/b"

// Display how many records are currently tagged
@ nMaxRow-3,27 SAY " Tagged records: "
@ nMaxRow-3,51 SAY "" color "n/b"
@ nMaxRow-2,28 SAY "" color "n/b"

// And now the help line
@ nMaxRow,2 SAY " <Space> - Tag Record   <F2> - Show Tagged   <Enter> - Edit  " +;
                " <Esc> - Exit  "

// Freeze the "tag" column to the left
oBrow:freeze := 1

// Begin browse in column two
oBrow:colPos := 2

lKeyWaiting := .F.
lBrowse := TRUE

DO WHILE (lBrowse)

  // Display how many records are currently tagged
  @ nMaxRow-3,27 SAY " Tagged records: " + Str( Sx_KeyCount("TEMP"), 6 ) + " "

  // Don't let anyone edit the "tag" column
  oBrow:colPos := iif( oBrow:colPos == 1, 2, oBrow:colPos )

  // If no key waiting then stabilize
  IF (!lKeyWaiting)
    DO WHILE (!oBrow:stabilize())
      // Interrupt stabilization if a key is hit
      IF ((nKey := Inkey()) != 0)
        lKeyWaiting := .T.
        EXIT
      ENDIF
    ENDDO
  ENDIF

  // Get a keystroke
  nKey := Inkey(0)

  // Handle the keystroke
  DO CASE
    CASE (nKey == K_ENTER)
      SetCursor(1)
      xOldKey := IF( Empty(IndexKey()), NIL, &(IndexKey()) )
      nCol := oBrow:getColumn(oBrow:colPos)
      oGet := GetNew(Row(), Col(), nCol:block, nCol:heading,, oBrow:colorSpec)
      ReadModal( {oGet} )
      xNewKey := IF( Empty(IndexKey()), NIL, &(IndexKey()) )
      IF xNewKey != xOldKey
        oBrow:refreshAll()
      ELSE
        oBrow:refreshCurrent()
      ENDIF
      SetCursor(0)

    CASE (nKey == K_DOWN)
      oBrow:down()

    CASE (nKey == K_UP)
      oBrow:up()

    CASE (nKey == K_PGDN)
      oBrow:pageDown()

    CASE (nKey == K_PGUP)
      oBrow:pageUp()

    CASE (nKey == K_CTRL_PGUP)
      oBrow:goTop()

    CASE (nKey == K_CTRL_PGDN)
      oBrow:goBottom()

    CASE (nKey == K_RIGHT)
      oBrow:right()

    CASE (nKey == K_LEFT)
      oBrow:left()

    CASE (nKey == K_HOME)
      oBrow:home()

    CASE (nKey == K_END)
      oBrow:end()

    CASE (nKey == K_CTRL_LEFT)
      oBrow:panLeft()

    CASE (nKey == K_CTRL_RIGHT)
      oBrow:panRight()

    CASE (nKey == K_CTRL_HOME)
      oBrow:panHome()

    CASE (nKey == K_CTRL_END)
      oBrow:panEnd()

    CASE (nKey == K_F2)
      // If tagged records are already being displayed, set index to LAST tag
      IF Sx_IndexName() == "TEMP" + OrdBagExt()
        SET TAG TO last
        @ MaxRow(),38 say "Tagged"
        dbGoTop()
      ELSE
        SET TAG TO temp
        @ MaxRow(),38 say "All   "
        dbGoTop()
      ENDIF
      oBrow:refreshAll()

    // SPACE - "tag/untag" this record
    CASE (nKey == K_SPACE)
      // If already viewing tagged records...
      nRec := RecNo()
      IF Sx_IndexName() == "TEMP" + OrdBagExt()
        dbSkip()                         //  move to the next record and
        nRec := RecNo()                  //  store it's record number, then
        dbSkip(-1)                       //  go back to the previous record
        Sx_KeyDrop()                     //  and untag it (drop the key).
        dbGoto( nRec )                   //  Reposition the record pointer
        oBrow:refreshAll()               //  and repaint the screen.
      ELSE
        SET TAG TO temp            // TEMP index
        IF Sx_FindRec( nRec )      // See if this record is already "tagged"
          Sx_KeyDrop()             // If so, untag the record (drop key)
          SET TAG TO last
        ELSE
          SET TAG TO last           // If not already tagged,
          dbGoto( nRec )            //  make sure were still on that rec
          Sx_KeyAdd( "TEMP" )       //  and add a key into the TEMP index
        ENDIF
      oBrow:refreshCurrent()
      ENDIF

    CASE (nKey == K_ESC)
      lBrowse := .F.

  ENDCASE

  lKeyWaiting := .F.             // Reset key waiting flag

ENDDO

CLOSE ALL
SetCursor( nCurs )
SetColor( cColor )
SetBlink( lBlink )
CLS

Return(NIL)


//--------------------------------//
FUNCTION IsTagged( oBrow )

LOCAL nRec := RecNo(), cRetVal := " "

IF Sx_KeyCount( "TEMP" ) == 0
  Return( cRetVal )
ENDIF

IF Sx_IndexName() == "TEMP" + OrdBagExt()
  Return( "" )
ENDIF

SET TAG TO temp
IF Sx_FindRec( nRec )
  cRetVal := ""
ENDIF
SET TAG TO last
dbGoto( nRec )
oBrow:refreshCurrent()

Return( cRetVal )

