/*
  ķ
                                                                           
   Program...: RYOFILT.PRG                                                 
   Author....: Team SuccessWare                                            
   Notice....: Copyright 1993-95 - SuccessWare 90, Inc.                    
   Purpose...: Demonstrates using MachSIx's Roll-Your-Own (RYO)            
               filters.                                                    
                                                                           
   Compile...: CLIPPER ryofilt /n/w /dCDX  (for SIXCDX)                    
           or: CLIPPER ryofilt /n/w /dNSX  (for SIXNSX)                    
                                                                           
   Link......: BLINKER FI ryofilt, sixtools @six3                          
                                                                           
   NOTES:  This program requires a TEST.DBF file of any size, which must   
           be created with our DBCRE8.EXE utility.  For example, to        
           create a TEST.DBF file with 1000 records, you would type this   
           at the DOS command-line:                                        
                                                                           
               C:\> DBCRE8 1000                                            
                                                                           
  Ľ
*/

#ifdef CDX
  #include "SIXCDX.CH"
  #define RDDNAME "SIXCDX"
#else
  #include "SIXNSX.CH"
  #define RDDNAME "SIXNSX"
#endif

#include "inkey.ch"
#include "MACHSIX.CH"

#define TRUE  .T.
#define FALSE .F.

#define UL_ROW  2
#define UL_COL  3
#define LR_ROW  MaxRow()-6
#define LR_COL  MaxCol()-3


FUNC RYOFilt()

LOCAL oBrow, lBrowse, nKey, nX, xOldKey, xNewKey, lBlink
LOCAL cColor, nCol, oGet, nMaxRow := MaxRow()
LOCAL nRec := 0, nCurs := SetCursor(0), lKeyWaiting := .F.
LOCAL lShowTagged:=.f., nHandle := 0

CLS

// Make sure TEST.DBF exists
IF !file("TEST.DBF")
  Alert( "TEST.DBF not found!    Use the DBCRE8.EXE;" +;
         "utility to create one before running this;" +;
         "program.  For example: DBCRE8 1000       " )
  QUIT
ENDIF

SET SCOREBOARD off

// Save Blink bit status and Colors
lBlink := SetBlink(FALSE)
cColor := SetColor("w+/b")
CLS

// Open database using the SIx Driver
USE TEST VIA RDDNAME

//............................................Create empty RYO filter
nHandle := m6_NewFilter()

// Make shadow
SetColor("w/n")
@ UL_ROW+1, UL_COL+1 CLEAR TO LR_ROW+1, LR_COL+1

// Create a TBrowse database object
oBrow := TBrowseDB( UL_ROW, UL_COL, LR_ROW, LR_COL )
oBrow:colorSpec := "N/W*, N/BG*,, B/BG*, B/W*, B/BG*, R/W*, B/R*"
SetColor( oBrow:colorSpec )

// Change the heading separator
oBrow:headSep := " "

// Add one column for each field
oBrow:addColumn(TBColumnNew("", {|| _m6IsTagged( oBrow, nHandle )} ))
FOR nX := 1 TO FCount()
    oBrow:addColumn(TBColumnNew(FieldName(nX), FieldBlock(FieldName(nX))))
NEXT

// Draw the title
@ 0,24 SAY " Roll-Your-Own (RYO) Filter Demo "
@ 0,56 SAY "" color "n/b"
@ 1,25 SAY "" color "n/b"

// Display how many records are currently tagged
@ nMaxRow-3,27 SAY " Tagged records: "
@ nMaxRow-3,51 SAY "" color "n/b"
@ nMaxRow-2,28 SAY "" color "n/b"

// And now the help line
@ nMaxRow,0 SAY " [Space] - Tag Record  [F2] - Show Tagged  [F3] - Reverse  [Enter] - Edit " +;
                " [Esc] "

// Freeze the "tag" column to the left
oBrow:freeze := 1

// Begin browse in column two
oBrow:colPos := 2

lKeyWaiting := .F.
lBrowse := TRUE

DO WHILE (lBrowse)

  // Display how many records are currently tagged
  @ nMaxRow-3,27 SAY " Tagged records: " + Str( m6_FiltCount( nHandle ), 6 ) + " "

  // Don't let anyone edit the "tag" column
  oBrow:colPos := iif( oBrow:colPos == 1, 2, oBrow:colPos )

  // If no key waiting then stabilize
  IF (!lKeyWaiting)
    DO WHILE (!oBrow:stabilize())
      // Interrupt stabilization if a key is hit
      IF ((nKey := Inkey()) != 0)
        lKeyWaiting := .T.
        EXIT
      ENDIF
    ENDDO
  ENDIF

  // Get a keystroke
  nKey := Inkey(0)

  // Handle the keystroke
  DO CASE
    CASE (nKey == K_ENTER)
      SetCursor(1)
      xOldKey := IF( Empty(IndexKey()), NIL, &(IndexKey()) )
      nCol := oBrow:getColumn(oBrow:colPos)
      oGet := GetNew(Row(), Col(), nCol:block, nCol:heading,, oBrow:colorSpec)
      ReadModal( {oGet} )
      xNewKey := IF( Empty(IndexKey()), NIL, &(IndexKey()) )
      IF xNewKey != xOldKey
        oBrow:refreshAll()
      ELSE
        oBrow:refreshCurrent()
      ENDIF
      SetCursor(0)

    CASE (nKey == K_DOWN)
      oBrow:down()

    CASE (nKey == K_UP)
      oBrow:up()

    CASE (nKey == K_PGDN)
      oBrow:pageDown()

    CASE (nKey == K_PGUP)
      oBrow:pageUp()

    CASE (nKey == K_CTRL_PGUP)
      oBrow:goTop()

    CASE (nKey == K_CTRL_PGDN)
      oBrow:goBottom()

    CASE (nKey == K_RIGHT)
      oBrow:right()

    CASE (nKey == K_LEFT)
      oBrow:left()

    CASE (nKey == K_HOME)
      oBrow:home()

    CASE (nKey == K_END)
      oBrow:end()

    CASE (nKey == K_CTRL_LEFT)
      oBrow:panLeft()

    CASE (nKey == K_CTRL_RIGHT)
      oBrow:panRight()

    CASE (nKey == K_CTRL_HOME)
      oBrow:panHome()

    CASE (nKey == K_CTRL_END)
      oBrow:panEnd()

    CASE (nKey == K_F3)

      m6_FiltInverse( nHandle )

      oBrow:refreshALL()

    CASE (nKey == K_F2)

      lShowTagged := !lShowTagged

      //...................if showing tagged records, activate the filter
      IF lShowTagged
        m6_SetAreaFilter( nHandle )
        @ MaxRow(),35 say "All   "
        dbGoTop()
        skip

      //.......................grab a handle to the filter, then clear it
      ELSE
        nHandle := m6_GetAreaFilter()
        m6_ChgOwner( nHandle )
        SET FILTER TO
        @ MaxRow(),35 say "Tagged"
        dbGoTop()
      ENDIF

      oBrow:refreshAll()

    // SPACE - "tag/untag" this record
    CASE (nKey == K_SPACE)
      nRec := RecNo()

      //.......................................Toggle status of this record
      m6_FiltChgRec( nHandle, nRec )

      if lShowTagged
         oBrow:refreshALL()
      else
        oBrow:refreshCurrent()
      endif

    CASE (nKey == K_ESC)
      lBrowse := .F.

  ENDCASE

  lKeyWaiting := .F.             // Reset key waiting flag

ENDDO

CLOSE ALL
SetCursor( nCurs )
SetColor( cColor )
SetBlink( lBlink )
CLS

Return(NIL)


//--------------------------------//
FUNCTION _m6IsTagged( oBrow, nHandle )

LOCAL nRec := RecNo()

//.........................................Is this record set in the filter ?
if m6_IsFiltRec( nHandle,  nRec )
  return( "" )
else
  return ( " " )
endif

Return( NIL )
