/*
  ķ
                                                                         
   Program...: SCOPE.PRG                                                 
   Author....: Team SuccessWare                                          
   Notice....: Copyright 1993-95 - SuccessWare 90, Inc.                  
   Purpose...: Demonstrates using the SIx Driver's index SCOPEs to limit 
               the range of records visible in a TBrowse().              
                                                                         
   Compile...: CLIPPER scope /n/w /dCDX  (for SIXCDX)                    
           or: CLIPPER scope /n/w /dNSX  (for SIXNSX)                    
                                                                         
   Link......: BLINKER FI scope, sixtools @six3                          
                                                                         
   NOTES:  This program requires a TEST.DBF file of any size, which must 
           be created with our DBCRE8.EXE utility.  For example, to      
           create a TEST.DBF file with 1000 records, you would type this 
           at the DOS command-line:                                      
                                                                         
               C:\> DBCRE8 1000                                          
                                                                         
  Ľ
*/


#ifdef CDX
  #include "SIXCDX.CH"
  #define RDDNAME "SIXCDX"
  #define INDEXEXT ".CDX"
#else
  #include "SIXNSX.CH"
  #define RDDNAME "SIXNSX"
  #define INDEXEXT ".NSX"
#endif

#include "inkey.ch"

#define TRUE  .T.
#define FALSE .F.

#define UL_ROW  2
#define UL_COL  3
#define LR_ROW  MaxRow()-6
#define LR_COL  MaxCol()-3

#define S_TOP     0
#define S_BOTTOM  1

//-----------------------------------------------------------------------//
FUNC ScopeDemo()

LOCAL oBrow, lBrowse := TRUE, nKey, nX, xOldKey, xNewKey, nInScope
LOCAL cColor, nCol, oGet, lBlink, nCurs, nLastRow := MaxRow(), nRow
LOCAL lKeyWaiting := .F.
FIELD LAST

// Make sure TEST.DBF exists
IF !file("TEST.DBF")
  Alert( "TEST.DBF not found!    Use the DBCRE8.EXE;" +;
         "utility to create one before running this;" +;
         "program.  For example: DBCRE8 1000       " )
  QUIT
ENDIF

SET SCOREBOARD off

// Save Blink bit status and Colors
lBlink := SetBlink(FALSE)
cColor := SetColor( "w+/b" )
CLS

// Delete TEST.CDX if it exists
IF File( "TEST" + INDEXEXT )
  fErase( "TEST" + INDEXEXT )
ENDIF

// Open database using the SIx Driver
USE TEST VIA RDDNAME

// Create LAST tag
INDEX ON last TAG last EVAL Odometer() EVERY 10
GO TOP

// Create "shadow"
SetColor( "w/n" )
@ UL_ROW+1, UL_COL+1 CLEAR TO LR_ROW+1, LR_COL+1

// Create a TBrowse database object
oBrow := TBrowseDB( UL_ROW, UL_COL, LR_ROW, LR_COL )
oBrow:colorSpec := "N/W*, N/BG*,, B/BG*, B/W*, B/BG*, R/W*, B/R*"
oBrow:headSep := " "

// Add one column for each field
FOR nX := 1 TO FCount()
  oBrow:addColumn(TBColumnNew(FieldName(nX), FieldBlock(FieldName(nX))))
NEXT

// Turn off the cursor
nCurs := SetCursor(0)

// Draw the title
SetColor( oBrow:colorSpec )
@ 0,32 SAY " SCOPE EXAMPLE "
@ 0,47 SAY "" color "n/b"
@ 1,33 SAY "" color "n/b"

// And now the help line
@ nLastRow,12 SAY " <F10> - Change Scope   <Enter> - Edit   <Esc> - Exit "
@ nLastRow-3,26 say " Records In Scope: "
@ nLastRow-3,45 say Str( Sx_KeyCount(), 6 ) + " " color "b/w*"
@ nLastRow-3,52 SAY "" color "n/b"
@ nLastRow-2,27 SAY "" color "n/b"

Sx_Skip0Refresh(TRUE)

DO WHILE (lBrowse)

  // If no key waiting then stabilize
  IF (!lKeyWaiting)
    DO WHILE (!oBrow:stabilize())
      // Interrupt stabilization if a key is hit
      IF ((nKey := Inkey()) != 0)
        lKeyWaiting := .T.
        EXIT
      ENDIF
    ENDDO
  ENDIF

  // Make sure at least one record is in scope;  Update display
  nInScope := Sx_KeyCount()
  nRow := Row()
  nCol := Col()
  @ nLastRow-3, 45 say Str( nInScope, 6 ) + " " color "b/w*"
  IF nInScope == 0
    @ UL_ROW+2, UL_COL CLEAR TO UL_ROW+2, LR_COL
  ENDIF
  DevPos( nRow, nCol )

  // Get a keystroke
  nKey := Inkey(0)

  // If no records in scope, only allow F10 and ESC keys to be used
  IF ((nInScope == 0) .and. (nKey != K_F10 .and. nKey != K_ESC))
    LOOP
  ENDIF

  // Handle the keystroke
  DO CASE
    CASE (nKey == K_ENTER)
      SetCursor(1)
      xOldKey := iif( Empty(OrdKey()), NIL, Sx_KeyData())
      nCol := oBrow:getColumn(oBrow:colPos)
      oGet := GetNew(Row(), Col(), nCol:block, nCol:heading,, oBrow:colorSpec)
      ReadModal( {oGet} )
      xNewKey := iif( Empty(OrdKey()), NIL, Sx_KeyData())
      IF xNewKey != xOldKey
        oBrow:refreshAll()
      ELSE
        oBrow:refreshCurrent()
      ENDIF
      SetCursor(0)

    CASE (nKey == K_DOWN)
      oBrow:down()

    CASE (nKey == K_UP)
      oBrow:up()

    CASE (nKey == K_PGDN)
      oBrow:pageDown()

    CASE (nKey == K_PGUP)
      oBrow:pageUp()

    CASE (nKey == K_CTRL_PGUP)
      oBrow:goTop()

    CASE (nKey == K_CTRL_PGDN)
      oBrow:goBottom()

    CASE (nKey == K_RIGHT)
      oBrow:right()

    CASE (nKey == K_LEFT)
      oBrow:left()

    CASE (nKey == K_HOME)
      oBrow:home()

    CASE (nKey == K_END)
      oBrow:end()

    CASE (nKey == K_CTRL_LEFT)
      oBrow:panLeft()

    CASE (nKey == K_CTRL_RIGHT)
      oBrow:panRight()

    CASE (nKey == K_CTRL_HOME)
      oBrow:panHome()

    CASE (nKey == K_CTRL_END)
      oBrow:panEnd()

    // F10 - change the scope
    CASE (nKey == K_F10)
      IF ChangeScope()       // Call our scope-change function
        oBrow:goTop()        // Position to top of new scope
        oBrow:refreshAll()
        DO WHILE (!oBrow:stabilize())
        ENDDO
      ENDIF

    CASE (nKey == K_ESC)
      lBrowse := FALSE

  ENDCASE

  lKeyWaiting := .F.             // Reset key waiting flag

ENDDO

CLOSE ALL
SetCursor( nCurs )
SetColor( cColor )
SetBlink( lBlink )
CLS


Return(NIL)


//-----------------------------------------------------------------------//
FUNC ChangeScope()

LOCAL cScrn, xScopeTop, xScopeBot, xTemp, nLen, nLastRow := MaxRow(),;
      nLastCol := MaxCol(), cPict, nScoped
LOCAL xOldTop, xOldBot
MEMVAR GetList

SET SCOREBOARD OFF

// Use old scopetop value as default if not NIL
IF ( xScopeTop := Sx_SetScope(S_TOP) ) == NIL
  // Otherwise, get key value from first record
  dbGoTop()
  xScopeTop := Sx_KeyData()
ENDIF

// Save TOP scope value
xOldTop := xScopeTop

// Use old scopebottom value as default if not NIL
IF ((xScopeBot := Sx_SetScope(S_BOTTOM)) == NIL)
  // Otherwise, get key value from last record
  dbGoBottom()
  xScopeBot := Sx_KeyData()
ENDIF

// Save BOTTOME scope value
xOldBot := xScopeBot

// Pad to full key width if character
IF ValType( xScopeTop ) == "C"
  nLen := len( Sx_KeyData() )
  xScopeTop := PADR( xScopeTop, nLen )
  xScopeBot := PADR( xScopeBot, nLen )
ENDIF

// Save the current screen
cScrn := SaveScreen( nLastRow-4, 0, nLastRow, nLastCol )

// Clear a chunk 'o screen
Scroll( nLastRow-3, 0, nLastRow, nLastCol, 0 )
@ nLastRow-3, 0 TO nLastRow-3, nLastCol

// Turn on the cursor
SetCursor(1)

// Show the expression
@ nLastRow-2,1 SAY "Index Expression..: " + OrdKey(0)

// Get new values
cPict := "@K !" + Repl( "X", len( xScopeTop )-1 )
@ nLastRow-1,1 SAY "Scope Top Value...:" GET xScopeTop PICT cPict
cPict := "@K !" + Repl( "X", len( xScopeBot )-1 )
@ nLastRow,1 SAY "Scope Bottom Value:" GET xScopeBot PICT cPict
READ

// Turn the cursor back off
SetCursor(0)

// Restore the screen
RestScreen( nLastRow-4, 0, nLastRow, nLastCol, cScrn )

// If user pressed ESC then blaze outahere!
IF LastKey() == K_ESC
  Return(FALSE)
ENDIF

// Make sure values are in the correct position
IF xScopeTop > xScopeBot
  xTemp     := xScopeTop
  xScopeTop := xScopeBot
  xScopeBot := xTemp
ENDIF

// Trim char scope vals so a partial match succeeds
IF ValType( xScopeTop ) == "C"
  xScopeTop := Trim( xScopeTop )
  xScopeBot := Trim( xScopeBot )
ENDIF

// Set (or clear) the new scope values
IF Empty( xScopeTop )
  Sx_ClrScope( S_TOP )
ELSE
  Sx_SetScope( S_TOP, xScopeTop )
ENDIF

IF Empty( xScopeBot )
  Sx_ClrScope( S_BOTTOM )
ELSE
  Sx_SetScope( S_BOTTOM, xScopeBot )
ENDIF

// Make sure at least one record is in SCOPE
IF Sx_KeyCount() == 0
  MyAlert("No Records Are In SCOPE!;;" +;
        "Restoring previous SCOPE..." )
  Sx_SetScope( S_TOP, xOldTop )
  Sx_SetScope( S_BOTTOM, xOldBot )
ENDIF

Return(TRUE)

