/*
  ķ
                                                                         
   Program...: SCRLLBAR.PRG                                              
   Author....: Team SuccessWare                                          
   Notice....: Copyright 1992-95 - SuccessWare 90, Inc.                  
   Purpose...: Demonstrates a simple TBrowse that uses the Sx_KeyNo()    
               and Sx_KeyCount() functions to create a scroll bar on the 
               right side of the browse window, indicating the relative  
               position within the index.                                
                                                                         
   Compile...: CLIPPER scrllbar /n/w /dCDX   (for SIXCDX)                
           or: CLIPPER scrllbar /n/w /dNSX   (for SIXNSX)                
                                                                         
   Link......: BLINKER FI scrllbar,sixcdx @six3  (for SIXCDX)            
           or: BLINKER FI scrllbar,sixnsx @six3  (for SIXNSX)            
                                                                         
   NOTES:  This program requires a TEST500.DBF of any size to be created 
           with the DBCRE8.EXE utility.  For example:                    
                                                                         
             > DBCRE8 500 /fTEST500                                      
                                                                         
  Ľ
*/

#ifdef CDX
  #include "SIXCDX.CH"
  #define RDDNAME "SIXCDX"
#else
  #include "SIXNSX.CH"
  #define RDDNAME "SIXNSX"
#endif

#include "INKEY.CH"
#include "BOX.CH"

FUNC ScrollBar()

LOCAL oBrow, lBrowse, lKeyWaiting, nKey, i
LOCAL nMaxRow := MaxRow()
LOCAL nMaxCol := MaxCol()
LOCAL cOldColor := SetColor("bg+/n"), lBlink := SetBlink(.F.)
MEMVAR GetList
FIELD last

// Make sure TEST500.DBF exists
IF !file("TEST500.DBF")
  Alert( "TEST500.DBF not found!    Use the DBCRE8.EXE;" +;
         "utility to create one before running this;" +;
         "program.  For example: DBCRE8 500 /fTEST500" )
  QUIT
ENDIF

// Open database using the SIx Driver
USE TEST500 VIA RDDNAME
CLS
? "Building Index: "
INDEX ON last TO last EVAL QQOut(".") EVERY 10
?? "DONE!"

// Adjust last row if not enough recs to fill screen
nMaxRow := iif( RecCount() < nMaxRow-3, RecCount()+3, nMaxRow)

// Create a TBrowse database object using full screen
oBrow := TBrowseDB( 3, 3, nMaxRow-1, nMaxCol-1 )
oBrow:colorSpec := "B/W*, N/BG*,, B/BG*, B/W*, B/BG*, R/W*, B/R*"
SetColor( oBrow:colorSpec )
CLS

// Change the heading separator
oBrow:headSep := " "

// Add one column for each field
FOR i := 1 TO FCOUNT()
  oBrow:addColumn(TBColumnNew(FieldName(i), FieldBlock(FieldName(i))))
NEXT

// Turn off the cursor
SetCursor(0)

// Draw a box
@ 0,0,nMaxRow,nMaxCol BOX B_DOUBLE+" " color "r/w*"
@ 2,0 say "" + Repl("",nMaxCol-1) + "" color "r/w*"

// Draw the title
@ 0,30 SAY " SCROLL BAR EXAMPLE " color "b/w"

@ 1, 4 SAY "Key Position: " color "b/w*"

// Now a line to separate the scroll bar
@ 5, 2 TO nMaxRow-1, 2
@ 4, 1 SAY "" color "r/w*"
@ nMaxRow, 1 SAY "" color "r/w*"

// Stabilize the TBrowse
DO WHILE (!oBrow:stabilize())
ENDDO

// Initialize the scroll bar
ShowScrollBar(5,1,nMaxRow-1,1)

lKeyWaiting := .F.
lBrowse     := .T.

DO WHILE (lBrowse)

  // If no key waiting then stabilize
  IF (!lKeyWaiting)
    DO WHILE (!oBrow:stabilize())
      // Interrupt stabilization if a key is hit
      IF ((nKey := INKEY()) != 0)
        lKeyWaiting := .T.
        EXIT
      ENDIF
    ENDDO

    // Update the scroll bar (no parameters means update)
    ShowScrollBar()
  ENDIF

  // If there's no keystroke waiting then get one.
  IF (!lKeyWaiting)
    nKey := Inkey(0)
  ENDIF

  // Handle the keystroke
  DO CASE
    CASE (nKey == K_DOWN)
      oBrow:down()

    CASE (nKey == K_UP)
      oBrow:up()

    CASE (nKey == K_PGDN)
      oBrow:pageDown()

    CASE (nKey == K_PGUP)
      oBrow:pageUp()

    CASE (nKey == K_CTRL_PGUP)
      oBrow:goTop()

    CASE (nKey == K_CTRL_PGDN)
      oBrow:goBottom()

    CASE (nKey == K_RIGHT)
      oBrow:right()

    CASE (nKey == K_LEFT)
      oBrow:left()

    CASE (nKey == K_HOME)
      oBrow:home()

    CASE (nKey == K_END)
      oBrow:end()

    CASE (nKey == K_CTRL_LEFT)
      oBrow:panLeft()

    CASE (nKey == K_CTRL_RIGHT)
      oBrow:panRight()

    CASE (nKey == K_CTRL_HOME)
      oBrow:panHome()

    CASE (nKey == K_CTRL_END)
      oBrow:panEnd()

    CASE (nKey == K_ESC)
      lBrowse := .F.

  ENDCASE

  lKeyWaiting := .F.             // Reset key waiting flag

ENDDO

CLOSE ALL
SetColor( cOldColor )
SetBlink( lBlink )
CLS

Return(NIL)


//

/*
   ShowScrollBar() - This function displays a simple vertical "scroll bar"
                     for a TBrowse on the current table and index.  Call
                     ShowScrollBar() with scroll bar screen coordinates to
                     initialize it, then with no parameters to update it.
                     Static variables are used to keep track of screen
                     coordinates and step value.  Called from Main().
*/

FUNC ShowScrollBar(nTop, nLeft, nBottom, nRight)

  STATIC nBarTop, nBarLeft, nBarBottom, nBarRight, nBarWidth
  STATIC nKeyStep, nKeys
  LOCAL  nPos, nKeyNo := Sx_KeyNo()

  // Did we get coordinates?
  IF (PCOUNT() > 0)
    // Yes, initialize...

    // Set the bar coordinates
    nBarTop    := nTop
    nBarLeft   := nLeft
    nBarBottom := nBottom
    nBarRight  := nRight
    nBarWidth  := (nRight - nLeft) + 1

    // Get number of keys in current index
    nKeys := Sx_KeyCount()

    // Calculate the number of keys to skip before moving "elevator"
    nKeyStep := INT(nKeys / (nBarBottom - nBarTop))
  ENDIF

  // Clear the scroll bar
  Scroll(nBarTop,nBarLeft,nBarBottom,nBarRight,0)

  // Draw the "elevator"
  nPos := MIN(nBarTop + (nKeyNo / nKeyStep), nBarBottom)
  @ nPos,nBarLeft SAY REPLICATE('',nBarWidth) color "r/w*"

  @ 1, 18 say PadR( AllTrim( Str( nKeyNo )) + "/" +;
                    AllTrim( Str( nKeys )), 10 ) COLOR "r/w*"

Return(NIL)

