/*
   Mar 15, 1995

   CA-Clipper Source Code for DBF/CDX/NSX Diagnostic Utility

   Requires:
      CA-Clipper v 5.2x or above    (Computer Associates)
      SIx Driver 2.0 or above       (SuccessWare)

What-It-Is
------------------
   TESTINX compares DBF data with index data, record by record.
   Index attributes, comparison errors and out-of-sequence errors are
   recorded in <DBFname>.LOG.

What-It-Isn't
------------------
   This is NOT the source code to TESTCDX.EXE found on the SuccessWare BBS 
   (BabelBoard.) But it's close. This will test all of your DBF data against
   all of your index data but the rebuild index options are not included 
   here.
   
What-else?   
------------------
   In addition to comparing dbf data with index data, order is also checked. 
   Order amongst duplicate keys is checked as well by looking at recno(). 
   "Smith" - rec#1  comes before "Smith" rec#2. Strictly speaking, this is 
   not a requirement for an index. 
   
   For NSX indexes built prior to SIx 3.0 and CDX indexes built prior to SIx 
   2.0, record number order for duplicate keys was not a consideration. 

Build
------------------
Compile: Clipper TESTINX /m /n /w /dRDD<rddname>

     eg: Clipper TESTINX /m /n /w /dRDDSIXCDX

   Link: Blinker file testinx.obj lib six3.lib blinker incremental off

History
------------------------------------------------------------------------------
   03-15-95 - Initial Release

------------------------------------------------------------------------------
   SuccessWare 90, Inc.
   27349 Jefferson Avenue #110
   Temecula, CA 92590

   Voice: (909) 699-9657
   Fax  : (909) 699-5679
   BBS  : (909) 694-6891
   CIS  : 74774,2240   (GO SWARE)
*/

#ifdef RDDSIXNSX
   #include "sixnsx.ch"
   #define RDDNAME "SIXNSX"
#else
   #include "sixcdx.ch"
   #define RDDNAME "SIXCDX"
#endif

#define BLOCKIFY(cExpr) &("{||" + cExpr + "}")
#define DESCENDING_INDEX   5

// REQUEST or make a call to functions you might use in an index expression.
REQUEST SOUNDEX
REQUEST DESCEND

function testrec(cDbf,xNew )
   local oldOrd, nKeyrow, nDatarow, nTmpRow, cLogFIle, nMaxCol,aInf_, iInf_
   local tkeydata, tevalordkey, lToFile, nRec, bBlock, nOrds, p, nMaxRow
   local newkeyData, newEvalOrdkey, dateFormat, nAt
   local nDup   := 0   ,;
         maxDup := 0   ,;
         i      := 1   ,;
         nRow   := 0   ,;
         isNew  := .f.
   // beats REQUEST because some Clipper functions don't
   // get brought in even w/ REQUEST
   padl()
   padr()
   isLower()

   cls

   /* parameter check */
   if empty( cDbf )
      ?
      ? "TESTINX v 1.00.00 - DBF/INDEX Diagnostic Utility"
      ?
      ? "TESTINX compares DBF data with index key data contained in indexes"
      ? "created with SuccessWare's " + RDDNAME + " RDD for CA-Clipper. Index"
      ? "attributes as well as comparison errors and out-of-sequence errors"
      ? "are recorded in a log file named <DBFname>.LOG."
      ?
      ? "Syntax:   TESTINX <DBFname>[.DBF] [Options]"
      ? 
      ? "Options:  /x  Delete <DBFname>.LOG file. (Default is append to existing)"  
      ?
      ? "     Eg: To test MYDATA.DBF against the tags in MYDATA." + iif( RDDNAME=="SIXNSX", "NSX", "CDX")
      ? "         and append the log information to MYDATA.LOG file:"
      ? 
      ? "                        TESTINX MYDATA"   
      ? 
      ?
      ? "TESTINX.EXE build Info--"
      ? "   " + version()
      ? "   " + sx_version(3)
      ?
      ?
      quit
   endif

   nAt := at(".", cDbf)
   if nAt > 0 
      cLogFile := substr(cDbf, 1, nAt - 1) + ".LOG"
   else
      cLogFile := cDbf + ".LOG"
   endif

   if !empty(xNew)
      isNew :=  ( upper(xNew) == "/X"  .or. upper(xNew) == "-X" )
   endif

   if isNew
      set printer to (cLogFile)
   else
      set printer to (cLogFile) ADDITIVE
   endif

   /* end parameter check */
   
   // set display limits
   nMaxCol    := maxcol() - 10
   nMaxRow  := maxrow() - 7

   use (cDbf) via RDDNAME

   /* 
      You may need to add code that opens other-than-structural index
      files. You can use the SIx Driver's Triggers and .INI file functions
      to accomplish this generically. (Check out INIDEMO.PRG in your 
      SIx Driver samples subdirectory.)
   */

   ordsetfocus(i)
   
   toFile( nRow++,0, "Num Recs         :"+ str(lastrec()) )
   
   while !empty( ordname(i) )
      nOrds := sx_tagCount()
      toFile( nRow++,0, "=====================================================" )
      toFile( nRow++,0,padr("Bag Name          :"+ ordbagName() , nMaxCol) )
      toFile( nRow++,0,padr("Num Tags          :"+ str(nOrds, 4), nMaxCol) )
      toFile( nRow++,0, "" )
      aInf_ := sx_tagInfo(ordBagName())

      for p := i to (i+nOrds) - 1
         iInf_ := aInf_[(p-i) + 1]
         
         ordsetfocus(p)
         go top

         toFile( nRow++, 3,padr(str(ordnumber())+"", nMaxCol) )
         toFile( nRow++ ,3,padr( "Key Count         :" + str( sx_keycount() ), nMaxCol ) )
         toFile( nRow++ ,3,padr( "Tag Name          : " + ordname()         , nMaxCol )  )
         toFile( nRow++ ,3,padr( "Key               : " + ordkey()          , nMaxCol )  )
         
         nKeyRow  := nRow
         nDataRow := nRow + 1 
         
         @ nRow++,3 say "keydata           : " 
         @ nRow++,3 say "eval ordkey macro : " 
         
         bBlock := BLOCKIFY(ordKey())
         
         @ nKeyRow , 25 say space(nMaxCol)
         
         @ nDataRow, 25 say space(nMaxCol)
         
         tKeyData    := sx_keydata()
         tEvalOrdKey := bBlock:eval()
         
         while ! eof()
            
            if getenv("TESTINDEX")="SHOWALL" 
               toFile(nKeyRow ,0,  str(recno()) + " " + (newkeydata := sx_keydata()) + " " + (newEvalOrdKey := bBlock:eval()) )
               nKeyRow++
            else
               @ nKeyRow ,25 say newkeydata := sx_keydata()
               @ nDataRow,25 say newEvalOrdKey := bBlock:eval()
            endif   
            
            // descending check
            if !iInf_[DESCENDING_INDEX]
               if !( newKeydata >= tKeyData ) .or. !( newEvalOrdKey >= tEvalOrdKey )
                  toFile( nRow++, 3, "ERROR Key Order  at :" + str(recno()) )
               endif
            else
               // make sure next value is less than or equal to prev value
               //  for descending index 
               if !( newKeydata <= tKeyData ) .or. !( newEvalOrdKey <= tEvalOrdKey )
                  toFile( nRow++, 3, "ERROR Key Order  at :" + str(recno()) )
               endif
            endif
            
            tKeyData    := newKeyData 
            tEvalOrdKey := newEvalOrdKey
            
            
            nDup := 0
            
            if !(tkeydata == tEvalOrdKey)
               toFile( nRow++,  3,"ERROR Data/Key Mismatch at # :" + str(recno())) 
               toFile( nRow++,  3,"KEYDATA : " + toChar( tKeyData ) )
               toFile( nRow++ , 3,"DBFDATA : " + toChar( tEvalOrdKey ) )
               skip
            else
               
               nRec := recno()
               skip
               
               while !eof() .and. (sx_keydata() == tKeydata);
                     .and. (bBlock:eval() == tEvalOrdKey)
                  nDup++
                  
                  if !iInf_[DESCENDING_INDEX]
                     if nRec > recno()
                        toFile( nRow++, 3, "ERROR Rec Num Order - Next # is :" + str(recno()) )
                     endif
                  else
                     if nRec < recno()
                        toFile( nRow++, 3, "ERROR Rec Num Order - Next # is :" + str(recno()) )
                     endif
                  endif
                  
                  nRec := recno()
                  
                  skip
                  
                  if eof()
                     exit
                  endif
                  
               enddo
               
               
            endif
            
            if nDup > maxDup
               maxDup := nDup
            endif 
            
         enddo
         
         toFile( nRow++ ,3,padr( "Max Duplicate Keys: " + str(maxDup), nMaxCol ) )
         
         maxDup := 0
         if nRow > nMaxRow
            @ 4,0 clear to maxrow()+ 1, maxcol() + 1
            setpos( 4,0)
            nRow := 4
         endif
      next
      i := i + nOrds
      ordsetfocus(i)
   enddo
   use
   set device to printer
   setprc( prow() + 1, 0 )
   @ 0,0 say " " + chr(13)
   set device to screen
   SET PRINTER TO                                // Close the output file
   return nil

function ToFile(nRow, nCol, cText)
   SET DEVICE TO PRINTER 
   @ 0, 0 say cText
   SET DEVICE TO SCREEN
   setpos(nRow, nCol)
   ?? cText
   return nil

function toChar(xValue)
   local cType := valtype(xValue)
   do case
      case cType == 'L'
         return iif( xValue, "TRUE", "FALSE" )
      case cType == 'N'
         return str(xValue)
      case cType == 'C'
         return xValue
      case cType == 'D'
         return dtoc( xValue )
   endcase
   return nil

