/*
  ķ
                                                                         
   Program...: TRIGDEMO.PRG                                              
   Author....: Team SuccessWare                                          
   Notice....: Copyright 1993-95 - SuccessWare 90, Inc.                  
   Purpose...: Demonstrates using the SIx Driver's Database Event        
               Triggers.                                                 
                                                                         
   Compile...: CLIPPER trigdemo /n/w /dCDX  (for SIXCDX)                 
           or: CLIPPER trigdemo /n/w /dNSX  (for SIXNSX)                 
                                                                         
   Link......: BLINKER FI trigdemo, sixtools, sixcdx @six3  (for SIXCDX) 
           or: BLINKER FI trigdemo, sixtools, sixnsx @six3  (for SIXNSX) 
                                                                         
   NOTES:  This program requires a TEST.DBF file of any size, which must 
           be created with our DBCRE8.EXE utility.  For example, to      
           create a TEST.DBF file with 1000 records, you would type this 
           at the DOS command-line:                                      
                                                                         
               C:\> DBCRE8 1000                                          
                                                                         
  Ľ
*/


#ifdef CDX
  #include "SIXCDX.CH"
  #define RDDNAME "SIXCDX"
  #define INDEXEXT ".CDX"
#else
  #include "SIXNSX.CH"
  #define RDDNAME "SIXNSX"
  #define INDEXEXT ".NSX"
#endif

#include "INKEY.CH"

FUNC TrigDemo()

LOCAL cCode := "", lBlink := SetBlink(.F.), cColor := SetColor()
LOCAL nRow := MaxRow(), nCol := MaxCol()

CLS

MyAlert("This demo shows how the SIx Driver's Database;" +;
        "Event Triggers can be used to control various;" +;
        "actions within a given workarea.             ;;"+;
        "In this demo, triggers will be used to verify;" +;
        "that the required indexes exist after the DBF;" +;
        "has been opened.  If not, they will be built.;;"+;
        "Also, once the table is in the browser, all  ;" +;
        "edits, deletes, recalls, etc, will be handled;" +;
        "by an event trigger.                         ")

// Delete any previous TEST2.CDX file
ERASE test2.cdx

SetColor("b/w*")
@ 1,0 TO nRow-8,nCol
SetColor("w+/b")
@ nRow-7,0 TO nRow,nCol
SetColor("w+/n")
@ nRow-6,1 CLEAR TO nRow-1,nCol-1

SetColor("b/w*")
USE test VIA RDDNAME TRIGGER "CustomTrig"
browse( 1, 0, nRow-8, nCol )
CLOSE ALL

MyAlert( "Now that the index has been built,;" +;
         "Let's reopen the file and see how ;" +;
         "the triggers respond...           " )

SetColor("b/w*")
USE test VIA RDDNAME TRIGGER "CustomTrig"
browse( 1, 0, nRow-8, nCol )
CLOSE ALL
inkey(2)

SetBlink( lBlink )
SetColor( cColor )

Return(NIL)

//----------------------------------------------------------//
FUNC CustomTrig( nEvent, nArea, nFieldPos, xTrigVal )

LOCAL nKey := 0
FIELD last,age

DO CASE

  CASE nEvent == EVENT_PREUSE
    MessageBox( "EVENT_PREUSE", "Opening " + xTrigVal, .F. )

  CASE nEvent == EVENT_POSTUSE
    MessageBox( "EVENT_POSTUSE", "Looking for indexes...", .F. )
    MessageBox( "   TEST2" + INDEXEXT, "", .F. )
    IF file("TEST2" + INDEXEXT)
      MessageBox( "   Found", "Opening and Setting Active Tag To LAST...", .F. )
      SET INDEX TO test2        // Select TEST2 index
      SET TAG TO last           // Set LAST tag as active order
    ELSE
      MessageBox( "   Not Found", "Creating TAGs...", .F. )
      INDEX ON last TAG last OF TEST2 EVAL Odometer() EVERY 50
      INDEX ON age  TAG age  OF TEST2 EVAL Odometer() EVERY 50
      SET TAG TO last
    ENDIF
    MessageBox( "   Done!","(Try doing some DELETES and EDITS...)", .F. )

  CASE nEvent == EVENT_UPDATE

  CASE nEvent == EVENT_APPEND
    Return( MessageBox( "EVENT_APPEND", "Verifying...", .T. ))

  CASE nEvent == EVENT_DELETE
    Return( MessageBox( "EVENT_DELETE", "Verifying...", .T. ))

  CASE nEvent == EVENT_RECALL
    Return( MessageBox( "EVENT_RECALL", "Verifying...", .T. ))

  CASE nEvent == EVENT_PACK

  CASE nEvent == EVENT_ZAP

  CASE nEvent == EVENT_PUT
    Return( MessageBox( "EVENT_PUT", "Verifying...", .T. ))

  CASE nEvent == EVENT_GET

  CASE nEvent == EVENT_PRECLOSE
    IF ValType( xTrigVal )=="C"
      MessageBox( "EVENT_PRECLOSE", "Closing " + xTrigVal, .F. )
    ELSE
      MessageBox( "EVENT_PRECLOSE", "Closing All", .F. )
    ENDIF

  CASE nEvent == EVENT_POSTCLOSE
    MessageBox( "EVENT_POSTCLOSE", "Datafile(s) Closed", .F. )

  CASE nEvent == EVENT_PREMEMOPACK

  CASE nEvent == EVENT_POSTMEMOPACK

ENDCASE

Return( .T. )

//----------------------------------------------------------//
FUNC MessageBox( cStr1, cStr2, lVerify )

LOCAL nRow := MaxRow(), nCol := MaxCol(), cColor := SetColor("bg+/n")
LOCAL lRetVal := .T.

Scroll( nRow-6, 1, nRow-1, nCol-1, 1)
@ nRow-1, 2 SAY PadR( cStr1, 16 ) + ":" COLOR "gr+/n"
@ nRow-1,21 SAY cStr2 COLOR "bg+/n"
SetColor( cColor )

IF lVerify
  lRetVal := MyAlert( "Are you sure?", {"Yes","No"} )
ENDIF

Return( lRetVal )

